	.file	"vectors.s"

#include "asm68k.h"

/*****************************************************************************
 *
 * buserr & adderr:
 * Bus & Address error generate more information on the supervisor stack.
 * This information is stored in global variables and control is transferred
 * to exception().
 * Supporting text in MC68000 processor book...
 *	pgs 90-94, 302-303, 315-317 
 */
	.text
	.global buserr
buserr:
	mov.b	&2,d0				/* regD0 = 2, bus error. */
	jmp	busaddmore

	.global adderr
adderr:
	mov.b	&3,d0				/* regD0 = 3, address error. */
	jmp	busaddmore

	.global busaddmore
busaddmore:
	and.l	&0xff,d0			/* Store vector number (low 8 bits only). */
	mov.l	d0,(regtbl+D0)	
	mov.w	(a7),d0
	mov.l	d0,(regtbl+A0)	/* regA0: status information word. */
	mov.w	2(a7),d0
	mov.l	d0,(regtbl+A1)	/* regA1: access address high. */
	mov.w	4(a7),d0
	mov.l	d0,(regtbl+A2)	/* regA2: access address low. */
	mov.w	6(a7),d0
	mov.l	d0,(regtbl+A3)	/* regA3: instruction register. */
	mov.w	8(a7),d0
	mov.w	d0,(regtbl+SR)	/* regSR: status register. */
	mov.l	10(a7),d0
	mov.l	d0,(regtbl+PC)	/* regPC: program counter. */
 	jsr	exception				/* Go to C. */

/*****************************************************************************
 #
 # Other vectors:
 # The code beginning at vbase is used by execption.c as a table of
 # vector handlers where the size of each member is "vnext - vbase".
 # This scheme keeps the C-code that initializes the vector handlers to
 # a minimum ; but still provides full-and-unique coverage of every
 # possible exception on the 68k family.
 # THe function gotoexception is used to stuff values into regtbl[] for
 # use by the general purpose exception() function.
 */
	.global gotoexception
gotoexception:
	mov.w	&0x0700,sr
	mov.l	a0,(regtbl+A0)
	mov.l	a1,(regtbl+A1)
	mov.l	a2,(regtbl+A2)
	mov.l	a3,(regtbl+A3)
	mov.l	a4,(regtbl+A4)
	mov.l	a5,(regtbl+A5)
	mov.l	a6,(regtbl+A6)
	mov.l	d0,(regtbl+D0)
	mov.l	d1,(regtbl+D1)
	mov.l	d2,(regtbl+D2)
	mov.l	d3,(regtbl+D3)
	mov.l	d4,(regtbl+D4)
	mov.l	d5,(regtbl+D5)
	mov.l	d6,(regtbl+D6)
	mov.l	d7,(regtbl+D7)
	mov.l	sp,(regtbl+SS)
	mov.w	(sp),a0
	mov.l	a0,(regtbl+SR)
	mov.l	2(a7),a0
	mov.l	a0,(regtbl+PC)
	mov.l	usp,a0
	mov.l	a0,(regtbl+US)
 	jsr	exception	


/*****************************************************************************/

	.global	vbase, vnext, vbase128, vnext128
vbase:
	mov.b	&2,d0
	jmp gotoexception
vnext:
	mov.b	&3,d0
	jmp gotoexception
	mov.b	&4,d0
	jmp gotoexception
	mov.b	&5,d0
	jmp gotoexception
	mov.b	&6,d0
	jmp gotoexception
	mov.b	&7,d0
	jmp gotoexception
	mov.b	&8,d0
	jmp gotoexception
	mov.b	&9,d0
	jmp gotoexception
	mov.b	&10,d0
	jmp gotoexception
	mov.b	&11,d0
	jmp gotoexception
	mov.b	&12,d0
	jmp gotoexception
	mov.b	&13,d0
	jmp gotoexception
	mov.b	&14,d0
	jmp gotoexception
	mov.b	&15,d0
	jmp gotoexception
	mov.b	&16,d0
	jmp gotoexception
	mov.b	&17,d0
	jmp gotoexception
	mov.b	&18,d0
	jmp gotoexception
	mov.b	&19,d0
	jmp gotoexception
	mov.b	&20,d0
	jmp gotoexception
	mov.b	&21,d0
	jmp gotoexception
	mov.b	&22,d0
	jmp gotoexception
	mov.b	&23,d0
	jmp gotoexception
	mov.b	&24,d0
	jmp gotoexception
	mov.b	&25,d0
	jmp gotoexception
	mov.b	&26,d0
	jmp gotoexception
	mov.b	&27,d0
	jmp gotoexception
	mov.b	&28,d0
	jmp gotoexception
	mov.b	&29,d0
	jmp gotoexception
	mov.b	&30,d0
	jmp gotoexception
	mov.b	&31,d0
	jmp gotoexception
	mov.b	&32,d0
	jmp gotoexception
	mov.b	&33,d0
	jmp gotoexception
	mov.b	&34,d0
	jmp gotoexception
	mov.b	&35,d0
	jmp gotoexception
	mov.b	&36,d0
	jmp gotoexception
	mov.b	&37,d0
	jmp gotoexception
	mov.b	&38,d0
	jmp gotoexception
	mov.b	&39,d0
	jmp gotoexception
	mov.b	&40,d0
	jmp gotoexception
	mov.b	&41,d0
	jmp gotoexception
	mov.b	&42,d0
	jmp gotoexception
	mov.b	&43,d0
	jmp gotoexception
	mov.b	&44,d0
	jmp gotoexception
	mov.b	&45,d0
	jmp gotoexception
	mov.b	&46,d0
	jmp gotoexception
	mov.b	&47,d0
	jmp gotoexception
	mov.b	&48,d0
	jmp gotoexception
	mov.b	&49,d0
	jmp gotoexception
	mov.b	&50,d0
	jmp gotoexception
	mov.b	&51,d0
	jmp gotoexception
	mov.b	&52,d0
	jmp gotoexception
	mov.b	&53,d0
	jmp gotoexception
	mov.b	&54,d0
	jmp gotoexception
	mov.b	&55,d0
	jmp gotoexception
	mov.b	&56,d0
	jmp gotoexception
	mov.b	&57,d0
	jmp gotoexception
	mov.b	&58,d0
	jmp gotoexception
	mov.b	&59,d0
	jmp gotoexception
	mov.b	&60,d0
	jmp gotoexception
	mov.b	&61,d0
	jmp gotoexception
	mov.b	&62,d0
	jmp gotoexception
	mov.b	&63,d0
	jmp gotoexception
	mov.b	&64,d0
	jmp gotoexception
	mov.b	&65,d0
	jmp gotoexception
	mov.b	&66,d0
	jmp gotoexception
	mov.b	&67,d0
	jmp gotoexception
	mov.b	&68,d0
	jmp gotoexception
	mov.b	&69,d0
	jmp gotoexception
	mov.b	&70,d0
	jmp gotoexception
	mov.b	&71,d0
	jmp gotoexception
	mov.b	&72,d0
	jmp gotoexception
	mov.b	&73,d0
	jmp gotoexception
	mov.b	&74,d0
	jmp gotoexception
	mov.b	&75,d0
	jmp gotoexception
	mov.b	&76,d0
	jmp gotoexception
	mov.b	&77,d0
	jmp gotoexception
	mov.b	&78,d0
	jmp gotoexception
	mov.b	&79,d0
	jmp gotoexception
	mov.b	&80,d0
	jmp gotoexception
	mov.b	&81,d0
	jmp gotoexception
	mov.b	&82,d0
	jmp gotoexception
	mov.b	&83,d0
	jmp gotoexception
	mov.b	&84,d0
	jmp gotoexception
	mov.b	&85,d0
	jmp gotoexception
	mov.b	&86,d0
	jmp gotoexception
	mov.b	&87,d0
	jmp gotoexception
	mov.b	&88,d0
	jmp gotoexception
	mov.b	&89,d0
	jmp gotoexception
	mov.b	&90,d0
	jmp gotoexception
	mov.b	&91,d0
	jmp gotoexception
	mov.b	&92,d0
	jmp gotoexception
	mov.b	&93,d0
	jmp gotoexception
	mov.b	&94,d0
	jmp gotoexception
	mov.b	&95,d0
	jmp gotoexception
	mov.b	&96,d0
	jmp gotoexception
	mov.b	&97,d0
	jmp gotoexception
	mov.b	&98,d0
	jmp gotoexception
	mov.b	&99,d0
	jmp gotoexception
	mov.b	&100,d0
	jmp gotoexception
	mov.b	&101,d0
	jmp gotoexception
	mov.b	&102,d0
	jmp gotoexception
	mov.b	&103,d0
	jmp gotoexception
	mov.b	&104,d0
	jmp gotoexception
	mov.b	&105,d0
	jmp gotoexception
	mov.b	&106,d0
	jmp gotoexception
	mov.b	&107,d0
	jmp gotoexception
	mov.b	&108,d0
	jmp gotoexception
	mov.b	&109,d0
	jmp gotoexception
	mov.b	&110,d0
	jmp gotoexception
	mov.b	&111,d0
	jmp gotoexception
	mov.b	&112,d0
	jmp gotoexception
	mov.b	&113,d0
	jmp gotoexception
	mov.b	&114,d0
	jmp gotoexception
	mov.b	&115,d0
	jmp gotoexception
	mov.b	&116,d0
	jmp gotoexception
	mov.b	&117,d0
	jmp gotoexception
	mov.b	&118,d0
	jmp gotoexception
	mov.b	&119,d0
	jmp gotoexception
	mov.b	&120,d0
	jmp gotoexception
	mov.b	&121,d0
	jmp gotoexception
	mov.b	&122,d0
	jmp gotoexception
	mov.b	&123,d0
	jmp gotoexception
	mov.b	&124,d0
	jmp gotoexception
	mov.b	&125,d0
	jmp gotoexception
	mov.b	&126,d0
	jmp gotoexception
	mov.b	&127,d0
	jmp gotoexception
vbase128:
	mov.b	&128,d0
	jmp gotoexception
vnext128:
	mov.b	&129,d0
	jmp gotoexception
	mov.b	&130,d0
	jmp gotoexception
	mov.b	&131,d0
	jmp gotoexception
	mov.b	&132,d0
	jmp gotoexception
	mov.b	&133,d0
	jmp gotoexception
	mov.b	&134,d0
	jmp gotoexception
	mov.b	&135,d0
	jmp gotoexception
	mov.b	&136,d0
	jmp gotoexception
	mov.b	&137,d0
	jmp gotoexception
	mov.b	&138,d0
	jmp gotoexception
	mov.b	&139,d0
	jmp gotoexception
	mov.b	&140,d0
	jmp gotoexception
	mov.b	&141,d0
	jmp gotoexception
	mov.b	&142,d0
	jmp gotoexception
	mov.b	&143,d0
	jmp gotoexception
	mov.b	&144,d0
	jmp gotoexception
	mov.b	&145,d0
	jmp gotoexception
	mov.b	&146,d0
	jmp gotoexception
	mov.b	&147,d0
	jmp gotoexception
	mov.b	&148,d0
	jmp gotoexception
	mov.b	&149,d0
	jmp gotoexception
	mov.b	&150,d0
	jmp gotoexception
	mov.b	&151,d0
	jmp gotoexception
	mov.b	&152,d0
	jmp gotoexception
	mov.b	&153,d0
	jmp gotoexception
	mov.b	&154,d0
	jmp gotoexception
	mov.b	&155,d0
	jmp gotoexception
	mov.b	&156,d0
	jmp gotoexception
	mov.b	&157,d0
	jmp gotoexception
	mov.b	&158,d0
	jmp gotoexception
	mov.b	&159,d0
	jmp gotoexception
	mov.b	&160,d0
	jmp gotoexception
	mov.b	&161,d0
	jmp gotoexception
	mov.b	&162,d0
	jmp gotoexception
	mov.b	&163,d0
	jmp gotoexception
	mov.b	&164,d0
	jmp gotoexception
	mov.b	&165,d0
	jmp gotoexception
	mov.b	&166,d0
	jmp gotoexception
	mov.b	&167,d0
	jmp gotoexception
	mov.b	&168,d0
	jmp gotoexception
	mov.b	&169,d0
	jmp gotoexception
	mov.b	&170,d0
	jmp gotoexception
	mov.b	&171,d0
	jmp gotoexception
	mov.b	&172,d0
	jmp gotoexception
	mov.b	&173,d0
	jmp gotoexception
	mov.b	&174,d0
	jmp gotoexception
	mov.b	&175,d0
	jmp gotoexception
	mov.b	&176,d0
	jmp gotoexception
	mov.b	&177,d0
	jmp gotoexception
	mov.b	&178,d0
	jmp gotoexception
	mov.b	&179,d0
	jmp gotoexception
	mov.b	&180,d0
	jmp gotoexception
	mov.b	&181,d0
	jmp gotoexception
	mov.b	&182,d0
	jmp gotoexception
	mov.b	&183,d0
	jmp gotoexception
	mov.b	&184,d0
	jmp gotoexception
	mov.b	&185,d0
	jmp gotoexception
	mov.b	&186,d0
	jmp gotoexception
	mov.b	&187,d0
	jmp gotoexception
	mov.b	&188,d0
	jmp gotoexception
	mov.b	&189,d0
	jmp gotoexception
	mov.b	&190,d0
	jmp gotoexception
	mov.b	&191,d0
	jmp gotoexception
	mov.b	&192,d0
	jmp gotoexception
	mov.b	&193,d0
	jmp gotoexception
	mov.b	&194,d0
	jmp gotoexception
	mov.b	&195,d0
	jmp gotoexception
	mov.b	&196,d0
	jmp gotoexception
	mov.b	&197,d0
	jmp gotoexception
	mov.b	&198,d0
	jmp gotoexception
	mov.b	&199,d0
	jmp gotoexception
	mov.b	&200,d0
	jmp gotoexception
	mov.b	&201,d0
	jmp gotoexception
	mov.b	&202,d0
	jmp gotoexception
	mov.b	&203,d0
	jmp gotoexception
	mov.b	&204,d0
	jmp gotoexception
	mov.b	&205,d0
	jmp gotoexception
	mov.b	&206,d0
	jmp gotoexception
	mov.b	&207,d0
	jmp gotoexception
	mov.b	&208,d0
	jmp gotoexception
	mov.b	&209,d0
	jmp gotoexception
	mov.b	&210,d0
	jmp gotoexception
	mov.b	&211,d0
	jmp gotoexception
	mov.b	&212,d0
	jmp gotoexception
	mov.b	&213,d0
	jmp gotoexception
	mov.b	&214,d0
	jmp gotoexception
	mov.b	&215,d0
	jmp gotoexception
	mov.b	&216,d0
	jmp gotoexception
	mov.b	&217,d0
	jmp gotoexception
	mov.b	&218,d0
	jmp gotoexception
	mov.b	&219,d0
	jmp gotoexception
	mov.b	&220,d0
	jmp gotoexception
	mov.b	&221,d0
	jmp gotoexception
	mov.b	&222,d0
	jmp gotoexception
	mov.b	&223,d0
	jmp gotoexception
	mov.b	&224,d0
	jmp gotoexception
	mov.b	&225,d0
	jmp gotoexception
	mov.b	&226,d0
	jmp gotoexception
	mov.b	&227,d0
	jmp gotoexception
	mov.b	&228,d0
	jmp gotoexception
	mov.b	&229,d0
	jmp gotoexception
	mov.b	&230,d0
	jmp gotoexception
	mov.b	&231,d0
	jmp gotoexception
	mov.b	&232,d0
	jmp gotoexception
	mov.b	&233,d0
	jmp gotoexception
	mov.b	&234,d0
	jmp gotoexception
	mov.b	&235,d0
	jmp gotoexception
	mov.b	&236,d0
	jmp gotoexception
	mov.b	&237,d0
	jmp gotoexception
	mov.b	&238,d0
	jmp gotoexception
	mov.b	&239,d0
	jmp gotoexception
	mov.b	&240,d0
	jmp gotoexception
	mov.b	&241,d0
	jmp gotoexception
	mov.b	&242,d0
	jmp gotoexception
	mov.b	&243,d0
	jmp gotoexception
	mov.b	&244,d0
	jmp gotoexception
	mov.b	&245,d0
	jmp gotoexception
	mov.b	&246,d0
	jmp gotoexception
	mov.b	&247,d0
	jmp gotoexception
	mov.b	&248,d0
	jmp gotoexception
	mov.b	&249,d0
	jmp gotoexception
	mov.b	&250,d0
	jmp gotoexception
	mov.b	&251,d0
	jmp gotoexception
	mov.b	&252,d0
	jmp gotoexception
	mov.b	&253,d0
	jmp gotoexception
	mov.b	&254,d0
	jmp gotoexception
	mov.b	&255,d0
	jmp gotoexception

