/* reset.s:
 *
 * First bit of boot code run by the processor.
 *
 */

	.extern start
	.extern stkinit
	.extern MonStack
	.extern usr_exception_handler
	.extern interrupt_handler
	.extern break_handler
	.extern hw_exception_handler
	.global reset
	.global warmstart
	.global coldstart
	.global vector_table
	.global vector_table_end

#include "config.h"
#include "warmstart.h"
#include "cpu.h"

/* reset:
 * This is the entrypoint for the monitor regardless of its location in
 * memory.  If this is located at zero, then this will be a valid vector
 * table; however, it can be located anywhere (assuming BRAM based code
 * is used to do the basic MicroBlaze boot) because vinit() will copy
 * it to the address specified by RAM_VECTOR_TABLE (in config.h).
 */
reset:
vector_table:
	brai	coldstart
	.align 2
	brai	usr_exception_handler
	.align 2
	brai	interrupt_handler
	.align 2
	brai	break_handler
	.align 2
	brai	hw_exception_handler
	.align 2
	nop				/* Reserved for future use by MicroBlaze */
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
vector_table_end:
	
	/* This should be an offset of 0x50 from the start of the
	 * vector table; and can be used to identify this uMon image.
	 */
	.ascii "MicroMonitor"
	.balign 0x10

#include "moncomptr.S"
#include "etheraddr.S"
#include "alttfsdevtbl.S"
	
coldstart:
	addi	r5, r0, INITIALIZE
	bri		warmstart

/*********************************************************************
 *
 * warmstart:
 * A point callable by C, as warmstart(int type), where
 * 'type' is one of the values defined in warmstart.h.
 * The argument (type) is stored away into a register (R19) that will
 * not be touched by any functions that may be called between this point
 * and the gotoC point below.  Actually R19 is a register that will be
 * saved across function calls, so if it isn't used in this assembler, it
 * is safe to assume that even if it is used by C functions, it will be
 * saved on the stack and restored when they return.
 */
warmstart:
	add		r19, r0, r5
    
/* SPInit:
 * Establish a stack frame....
 * Initialize the MonStack area, then establish a valid stack pointer.
 */
SPInit:
	addi	r1, r0, MonStack
	addi	r1, r1, (MONSTACKSIZE+64)
	brlid	r15, stkinit
	nop
    
	addi	r1, r0, MonStack
	addi	r1, r1, (MONSTACKSIZE-16)

/* gotoC:
 * Jump to the start() function in the monitor which should never return.
 */
gotoC:
	/* - Retrieve the variable passed to warmstart and place it
	 *   whereever it needs to be so that the start() function in
	 *   'C' sees it as an argument.  Note that if FORCE_BSS_INIT
	 *   is defined, then start() will ignore this argument.
	 * - Branch to start().
	 */
	add		r5, r0, r19
	braid	start
	nop


/*********************************************************************
 *
 * getMSR:
 * Return the content of the MSR register.
 * Note: R5 is the first incoming parameter and R3 is the return value.
 */
	.align 2
	.global getMSR
getMSR:
	mfs		r3, rmsr
	rtsd	r15, 8
	nop

/*********************************************************************
 *
 * setMSR:
 * Set specified bits in MSR register and return new MSR value.
 */
	.align 2
	.global setMSR
setMSR:
	mfs		r3, rmsr
	or		r3, r3, r5
	mts		rmsr, r3
	rtsd	r15, 8
	nop

/*********************************************************************
 *
 * clrMSR:
 * Clear specified bits in MSR register and return new MSR value.
 */
	.align 2
	.global clrMSR
clrMSR:
	mfs		r3, rmsr
	andn	r3, r3, r5
	mts		rmsr,r3
	rtsd	r15, 8
	nop
