#include "config.h"
#if INCLUDE_ETHERNET
#include "stddefs.h"
#include "genlib.h"
#include "ether.h"
#include "mips.h"

#define SYS_MAC_ADDR_SIZE		6

#define PHY_ADDR				0
#define PHY_CONTROL_DEFAULT		0x3000

/* Min pkt size...
 *	64 is spec but 4 bytes will be taken automatically by FCS
 */
#define MIN_PKTSIZE				60

/* Max pkt size...
 *	accommodates worst case.
 */
#define MAX_PKTSIZE				2048

/* Number of TX buffers = number of RX buffers = 4
 */
#define TX_BUFCNT				4
#define RX_BUFCNT				4

#define	MACREG(base,offs)	(*((volatile ulong *)((ulong)(base) + offs) ))

#define SET(f) 		(1<<f)                 /* set(CSR0_RA) | set(CSR0_PR) */
#define SETV(v,f) 	(((unsigned int)(v)) << f)
#define CLR(f) 		(0xFFFFFFFF ^ (1<<f))  /* clr(CSR0_RA) & clr(CSR0_PR) */

/* Network device statistics. 
 */
typedef struct net_device_stats
{
	ulong   rx_packets;             /* total packets received       */
	ulong   tx_packets;             /* total packets transmitted    */
	ulong   rx_bytes;               /* total bytes received         */
	ulong   tx_bytes;               /* total bytes transmitted      */
	ulong   rx_errors;              /* bad packets received         */
	ulong   tx_errors;              /* packet transmit problems     */
	ulong   multicast;              /* multicast packets received   */
	ulong   collisions;
	ulong	tx_timeout_errors;

	/* detailed rx_errors: */
	ulong   rx_zero_length_errors;
	ulong   rx_buffer_length_errors;
	ulong   rx_length_errors;
	ulong   rx_pf_errors;
	ulong   rx_cr_errors;          /* recved pkt with crc error    */
	ulong   rx_db_errors;        /* recv'd frame alignment error */
	ulong   rx_fl_errors;         /* recv'r fifo overrun          */
	ulong   rx_mi_errors;

	/* detailed tx_errors */
	ulong   tx_fa_errors;
	ulong   tx_jt_errors;
	ulong   tx_nc_errors;
	ulong   tx_ls_errors;
	ulong   tx_ed_errors;
	ulong   tx_lc_errors;
	ulong   tx_ec_errors;
	ulong   tx_ur_errors;
} t_net_device_stats ;


/* Device context for a AU1000 LAN controller
 */
typedef struct MAC_AU1000_device
{
	/* pointer for the AU1000 LAN controller register set */
	ulong	*pAU1000CtrlRegs;

	/* pointer for the AU1000 LAN Enable register set */
	ulong	*pAU1000EnReg;

	/* pointer for the AU1000 LAN DMA register set */
	ulong	*pAU1000DMARegs;

	/* Rcv buffer Index */
	uchar	NextRcvBufferIndex;

	/* Tx buffer index */
	uchar	NextTxmBufferIndex;

	ulong	TxmBuffer[TX_BUFCNT];
	ulong	RcvBuffer[RX_BUFCNT];

	uchar	PhysicalAddress[SYS_MAC_ADDR_SIZE];

	/* network statistics */
	t_net_device_stats	status ;

} t_MAC_AU1000_device ;

#define MAC0_AU1000_BASE	0xB0500000 
#define MAC1_AU1000_BASE	0xB0510000

#define MAC0_AU1000_EN_BASE	0xB0520000
#define MAC1_AU1000_EN_BASE	0xB0520004

#define MAC0_AU1000_DMA_BASE 0xB4004000
#define MAC1_AU1000_DMA_BASE 0xB4004200

#define Au1500_PBUS1_OFFSET	0x01000000		/* offset between PBUS 1 & PBUS 0 */

#define MAC_CONTROL	0x0000 /* MAC Control      */
#define MAC_ADDRHI	0x0004 /* MAC Address High */
#define MAC_ADDRLO	0x0008 /* MAC Address Low  */
#define MAC_MCHASHH	0x000C /* Multicast UpAddr */
#define MAC_MCHASHL	0x0010 /* Multicast LoAddr */
#define MAC_MIICTRL	0x0014 /* MII Address      */
#define MAC_MIIDATA	0x0018 /* MII Data         */
#define MAC_FLOWCTL	0x001C /* Flow Control     */
#define MAC_VLAN1TG	0x0020 /* VLAN1 Tag        */
#define MAC_VLAN2TG	0x0024 /* VLAN2 Tag        */


#define MAC_ENABLE 0x0000

#define MAC_DMA_CHAN	 9  /* MAC DMA Channel (0 or 1)              */
#define MAC_DMA_TXRX	 8  /* MAC DMA TxRx (0-Transmit 1-Receive)   */
#define MAC_DMA_BUF	 	 4  /* MAC DMA Buffer offset (2 bits)		 */
#define MAC_DMA_REG	 	 2  /* MAC DMA Register offset
							00 - Status
							01 - Address/Enable
							10 - Length (transmit only)
							11 - Reserved					     	 */

#define DMA_RECEIVE		1
#define DMA_TRANSMIT	0

#define DMA_LENGTH		2
#define DMA_ADDRESS_EN	1
#define DMA_STATUS		0

#define MAC_CONTROL_RA 		31  	/* 0  Receive All incoming packets       */
#define MAC_CONTROL_EM		30  	/* 0  Data Buffers Endian Mode 1-BE 0-LE */
									/* 29-24 Reserved                        */
#define MAC_CONTROL_DO		23  	/* 0  Disable Receive Own (RC-0 DIS-1)   */
#define MAC_CONTROL_LM 		21  	/* 00 Loopback operating mode            */
									/* b00 - Normal Mode                     */
									/* b01 - Internal Loopback               */
									/* b10 - External Loopback               */
									/* b11 - Reserved                        */
#define MAC_CONTROL_F  		20  	/* 0  Full Duplex Mode (Full-1 Half-0)   */
#define MAC_CONTROL_PM 		19  	/* 0  Pass All Multicast                 */
#define MAC_CONTROL_PR 		18  	/* 1  Promiscuous Mode (PR-1)            */
#define MAC_CONTROL_IF 		17  	/* 0  Inverse Filtering                  */
#define MAC_CONTROL_PB 		16  	/* 0  Pass Bad Frames                    */
#define MAC_CONTROL_HO 		15  	/* 0  Hash Only Filtering Mode           */
									/* 14 Reserved                           */
#define MAC_CONTROL_HP 		13  	/* 0  Hash Perfect (Imperfect-1 Perf-0)  */
#define MAC_CONTROL_LC		12  	/* 0  Deferral Check                     */
#define MAC_CONTROL_DB		11  	/* 0  Disable Broadcast Frames           */
#define MAC_CONTROL_DR		10  	/* 0  Disable Retry                      */
									/* 09 Reserved                           */
#define MAC_CONTROL_AP		 8  	/* 0  Automatic  Pad Stripping           */
#define MAC_CONTROL_BL		 6  	/* 00 Back Off Limit                     */
#define MAC_CONTROL_DC   	 5		/* Deferral Check */
									/* 04 Reserved                           */
#define MAC_CONTROL_TE   	 3  	/* 0  Transmitter Enable                 */
#define MAC_CONTROL_RE   	 2  	/* 0  Receiver Enable                    */
									/* 01-00 Reserved                        */

			    				/* 31-16 Reserved                        */
#define MAC_MIICTRL_PHYA	11  /* 00000 5-bit Phy Address               */
#define MAC_MIICTRL_MREG	6  	/* 00000 5-bit MII Register              */
			    				/* 05-02 Reserved                        */
#define	MAC_MIICTRL_MWR		1  	/* 0 MII Write  (1-Write 0-Read)         */
#define	MAC_MIICTRL_MBUSY	0  	/* 0 MII Busy                            */

#define	MAC_MIICTRL_MBUSY_MASK	(1 << MAC_MIICTRL_MBUSY)


			    					/* 31-16 Reserved                        */
#define MAC_MIIDATA_DATA	 0  	/* FFFF 16-bit MII Data                  */


#define MAC_ENABLE_DMARESET		6	/* DMA (ae_b) Reset						 */
#define MAC_ENABLE_TXEN			5	/* Value is anded with TX_EN			 */
#define MAC_ENABLE_MIIENAB		4	/* 0-Assert 1-Deassert Rst_T/RxClkN		 */
#define MAC_ENABLE_NONCAC		3	/* Cacheable bit 0-cacheable 1-non-cache */
#define MAC_ENABLE_DISTOS		2	/* Disable Tossing 0-Toss 1-Disable Toss */
#define MAC_ENABLE_MACINIT		1	/* 0-> Resets MAC, 1-> Enables MAC		 */
#define MAC_ENABLE_MACCLK		0	/* New Mac Enable Bit Location 			 */

/* Global driver states - the numbers are arbitrary */
#define MAC_AU1000_DRIVER_IS_STOPPED   0x42
#define MAC_AU1000_DRIVER_IS_STARTED   0x44


#define MAC_DMA_ADDR		 5  /* 27 bits: Address field starting at bit 5 */
#define MAC_DMA_NEXTBUF		 2  /* 2 bits: next buffer to be used by DMA*/
#define MAC_DMA_DONE	 	 1  /* MAC DMA Done bit            	     	 */
#define MAC_DMA_ENABLE		 0  /* MAC DMA Enable bit        	     	 */

#define MAC_DMA_ADDR_MASK	0xFFFFFFE0
#define MAC_DMA_NEXTBUF_MASK (0x3 << MAC_DMA_NEXTBUF)

#define MAC_DMA_DONE_MASK	 0x02
#define MAC_DMA_ENABLE_MASK	 0x01
#define MAC_DMA_DISABLE_MASK ~MAC_DMA_ENABLE_MASK

#define MAC_DMA_READY_MASK	0x3


#define MAC_TRANSMIT_PR		31  /* Packet Retry								 */
			   					/* 30-14 Reserved                   	     */
#define MAC_TRANSMIT_CC		10  /* 4-bit Collision Count		 	 	     */
#define MAC_TRANSMIT_LO		 9  /* Late Collision Observed (retried)	     */
#define MAC_TRANSMIT_DF		 8  /* Deferred                 	    	 	 */
#define MAC_TRANSMIT_UR	 	 7  /* Data Under Run           	    	 	 */
#define MAC_TRANSMIT_EC 	 6  /* Excessive Collisions        	    	 	 */
#define MAC_TRANSMIT_LC 	 5  /* Late Collision           	    	 	 */
#define MAC_TRANSMIT_ED 	 4  /* Excessive Defferal        	    	 	 */
#define MAC_TRANSMIT_LS		 3  /* Loss of Carrier          	    	 	 */
#define MAC_TRANSMIT_NC 	 2  /* No Carrier               	    	 	 */
#define MAC_TRANSMIT_JT		 1  /* Jabber Timeout           	    	 	 */
#define MAC_TRANSMIT_FA		 0  /* Frame Abort              	    	 	 */

#define MAC_TRANSMIT_CC_MASK	(0xf << MAC_TRANSMIT_CC)
#define MAC_TRANSMIT_UR_MASK	(1 << MAC_TRANSMIT_UR)
#define MAC_TRANSMIT_EC_MASK	(1 << MAC_TRANSMIT_EC)
#define MAC_TRANSMIT_LC_MASK	(1 << MAC_TRANSMIT_LC)
#define MAC_TRANSMIT_ED_MASK	(1 << MAC_TRANSMIT_ED)
#define MAC_TRANSMIT_LS_MASK	(1 << MAC_TRANSMIT_LS)
#define MAC_TRANSMIT_NC_MASK	(1 << MAC_TRANSMIT_NC)
#define MAC_TRANSMIT_JT_MASK	(1 << MAC_TRANSMIT_JT)
#define MAC_TRANSMIT_FA_MASK	(1 << MAC_TRANSMIT_FA)

#define MAC_RECEIVE_MI		31  /* Missed Frame 			     			 */
#define MAC_RECEIVE_PF		30  /* Packet Filter 			    			 */
#define MAC_RECEIVE_FF		29  /* Filtering Fail 			    			 */
#define MAC_RECEIVE_BF		28  /* Broadcast Frame 			    			 */
#define MAC_RECEIVE_MF		27  /* MultiCast Frame 			    			 */
#define MAC_RECEIVE_UC 		26  /* Unsupprted Control Frame					 */
#define MAC_RECEIVE_CF 		25  /* Control Frame			    			 */
#define MAC_RECEIVE_LE 		24  /* Length Error 			    			 */
#define MAC_RECEIVE_V2		23  /* VLAN 2 Frame Check		    			 */
#define MAC_RECEIVE_V1		22  /* VLAN 1 Frame Check		    			 */
#define MAC_RECEIVE_CR  	21  /* CRC Error            		  			 */
#define MAC_RECEIVE_DB 		20  /* Dribble Bit          		  			 */
#define MAC_RECEIVE_ME		19  /* MII Error (MII_Rxer) 		  			 */
#define MAC_RECEIVE_FT		18  /* Frame Type (1-Ethernet 0-802.3)			 */
#define MAC_RECEIVE_CS		17  /* Collision Seen 			    			 */
#define MAC_RECEIVE_FL		16  /* Frame Too Long  			    			 */
#define MAC_RECEIVE_RF		15  /* Runt Frame        		    			 */
#define MAC_RECEIVE_WT		14  /* Watch Dog Timeout 		    			 */
#define MAC_RECEIVE_L		 0  /* 14 bit Frame Length 		    			 */

#define MAC_RECEIVE_MI_MASK	(1 << MAC_RECEIVE_MI)
#define MAC_RECEIVE_PF_MASK	(1 << MAC_RECEIVE_PF)
#define MAC_RECEIVE_FF_MASK	(1 << MAC_RECEIVE_FF)
#define MAC_RECEIVE_BF_MASK	(1 << MAC_RECEIVE_BF)
#define MAC_RECEIVE_MF_MASK	(1 << MAC_RECEIVE_MF)
#define MAC_RECEIVE_UC_MASK	(1 << MAC_RECEIVE_UC)
#define MAC_RECEIVE_CF_MASK	(1 << MAC_RECEIVE_CF)
#define MAC_RECEIVE_LE_MASK	(1 << MAC_RECEIVE_LE)
#define MAC_RECEIVE_V2_MASK	(1 << MAC_RECEIVE_V2)
#define MAC_RECEIVE_V1_MASK	(1 << MAC_RECEIVE_V1)
#define MAC_RECEIVE_CR_MASK	(1 << MAC_RECEIVE_CR)
#define MAC_RECEIVE_DB_MASK	(1 << MAC_RECEIVE_DB)
#define MAC_RECEIVE_ME_MASK	(1 << MAC_RECEIVE_ME)
#define MAC_RECEIVE_FT_MASK	(1 << MAC_RECEIVE_FT)
#define MAC_RECEIVE_CS_MASK	(1 << MAC_RECEIVE_CS)
#define MAC_RECEIVE_FL_MASK	(1 << MAC_RECEIVE_FL)
#define MAC_RECEIVE_RF_MASK	(1 << MAC_RECEIVE_RF)
#define MAC_RECEIVE_WT_MASK	(1 << MAC_RECEIVE_WT)
#define MAC_RECEIVE_L_MASK	(0x3fff << MAC_RECEIVE_L)

/* Global driver state */
static ulong MAC_AU1000_state;

/* Device context per minor device for this driver */
static t_MAC_AU1000_device   minor_device;

/* Dynamic allocated memory */
static void *lan_dma_buffers;

static long MAC_AU1000_allocate_buffers( t_MAC_AU1000_device *pdevice ) ; 
static long MAC_AU1000_stop( t_MAC_AU1000_device *pdevice ) ;            
static long MAC_AU1000_MII_init( t_MAC_AU1000_device *pdevice ) ;        
static long MAC_AU1000_MII_status( t_MAC_AU1000_device *pdevice ) ;        
static long MAC_AU1000_control_init( t_MAC_AU1000_device *pdevice ) ;
static long MAC_AU1000_DMA_init( t_MAC_AU1000_device *pdevice ) ;        
static void MAC_AU1000_dump_status( t_MAC_AU1000_device *pdevice ) ;

long MAC_AU1000_read(void);
long MAC_AU1000_write(uchar *buf,int len);

#define MEMBLOCKSIZE \
	(((MAX_PKTSIZE * TX_BUFCNT) + \
	  (MAX_PKTSIZE * RX_BUFCNT)) / 4)

static unsigned long memblock[ MEMBLOCKSIZE+64];
static uchar tx_buff[MAX_PKTSIZE];

static long
MAC_AU1000_allocate_buffers(t_MAC_AU1000_device *pdevice)
{
	int		mem_size;
	void	*Pa;
	ulong	i;

	if(lan_dma_buffers == 0)
	{
		/* allocate all RX and TX packets in one chunk on a 2K boundary. */
		mem_size	 = MAX_PKTSIZE * TX_BUFCNT
						+ MAX_PKTSIZE * RX_BUFCNT;

		Pa = memblock;
		while((int)Pa & 0x1f)
			Pa++;

		/* cache this reference */
		lan_dma_buffers = Pa ;
	}
	else
	{
		/* not first time */
		Pa = lan_dma_buffers;
	}
	
	/* Initialize buffer space */
	mem_size	 = MAX_PKTSIZE * TX_BUFCNT
					+ MAX_PKTSIZE * RX_BUFCNT;
	
	Pa = (void*) KSEG1((ulong)Pa);
	memset(Pa,0,mem_size);

	/* Init RX buffer pointers */
	for (i=0;i < RX_BUFCNT; i++ )
	{
		pdevice->RcvBuffer[i]	= PHYS((ulong) Pa);
		Pa += MAX_PKTSIZE ;
	}

	/* Init TX buffer pointers */
	for ( i=0; i < TX_BUFCNT; i++ )
	{
		pdevice->TxmBuffer[i]	= PHYS((ulong) Pa);
		Pa += MAX_PKTSIZE ;
	}

	return(0);
}

static long
MAC_AU1000_stop( t_MAC_AU1000_device *pdevice )			   
{
	int i;

	/* Clear active low Reset Bits and set DMA RESET (active high) */
	MACREG(pdevice->pAU1000EnReg,MAC_ENABLE ) = SET(MAC_ENABLE_DMARESET);
	
	/* Clear Enable for Tx DMA */

	for (i=0; i< TX_BUFCNT ; i++) {
		
		MACREG( pdevice->pAU1000DMARegs, 
			(SETV(DMA_TRANSMIT, MAC_DMA_TXRX) |	
			SETV(i,MAC_DMA_BUF)	|		/* i selects buffer to disable */
			SETV(DMA_ADDRESS_EN ,MAC_DMA_REG)))
			&= MAC_DMA_DISABLE_MASK;				
	}

	/* Clear Enable for Tx DMA */

	for (i=0; i< RX_BUFCNT ; i++) {
		
		MACREG(pdevice->pAU1000DMARegs, 
			(SETV(DMA_RECEIVE , MAC_DMA_TXRX) |		
			SETV(i,MAC_DMA_BUF)  | SETV(DMA_ADDRESS_EN ,MAC_DMA_REG)))
			&= MAC_DMA_DISABLE_MASK;			
	}

	MAC_AU1000_state = MAC_AU1000_DRIVER_IS_STOPPED;
	return(0);
}

static void
MAC_MII_WRITE( t_MAC_AU1000_device *pdevice, int reg, int val )
{
	/* check link status, spin here till done */
	while ( MACREG( pdevice->pAU1000CtrlRegs, MAC_MIICTRL) 
					& MAC_MIICTRL_MBUSY_MASK) ;

	MACREG(pdevice->pAU1000CtrlRegs, MAC_MIIDATA) = val;
	MACREG(pdevice->pAU1000CtrlRegs, MAC_MIICTRL) = (reg << 6) | 0x02;

	/* check link status, spin here till done */
	while ( MACREG( pdevice->pAU1000CtrlRegs, MAC_MIICTRL) 
					& MAC_MIICTRL_MBUSY_MASK) ;
}

static long
MAC_AU1000_MII_init( t_MAC_AU1000_device *pdevice )		   
{
	MAC_MII_WRITE(pdevice,0,0x3200);	// reset Autonegotiation
	MAC_MII_WRITE(pdevice,17,0xffc0);	// set LEDs to display correctly
	return(0);
}

static long
MAC_AU1000_control_init(t_MAC_AU1000_device *pdevice)
{
	extern unsigned char BinEnetAddr[];
	int i;
	
	for(i=0;i<6;i++) 
		pdevice->PhysicalAddress[i] = BinEnetAddr[i];

	/* Bring MAC out of Reset */

	/* Enable CLocks */
	MACREG(pdevice->pAU1000EnReg,MAC_ENABLE) =
		SET(MAC_ENABLE_MACCLK) | SET(MAC_ENABLE_DMARESET);

	/* Bring out of reset */

	MACREG(pdevice->pAU1000EnReg,MAC_ENABLE) =
		SET(MAC_ENABLE_TXEN) | SET(MAC_ENABLE_MIIENAB) |
		SET(MAC_ENABLE_MACINIT) | SET(MAC_ENABLE_MACCLK);

	/* Turn off promiscuous mode - other than that defaults should be OK */
	
#ifdef EB
	MACREG( pdevice->pAU1000CtrlRegs, MAC_CONTROL) = 
		SETV(0, MAC_CONTROL_PR) | SET(MAC_CONTROL_EM) | SET(MAC_CONTROL_DO);
#else
	MACREG( pdevice->pAU1000CtrlRegs, MAC_CONTROL) =
		SETV(0, MAC_CONTROL_PR) | SET(MAC_CONTROL_DO);  
#endif

	/* Program Address -
		The LSB of low address is the first bit to compare on receive
		so this needs to contain the first byte to be compared */

	MACREG( pdevice->pAU1000CtrlRegs, MAC_ADDRLO) =
		pdevice->PhysicalAddress[0] 	   |
		pdevice->PhysicalAddress[1] << 8   |
		pdevice->PhysicalAddress[2] << 16  |
		pdevice->PhysicalAddress[3] << 24;

	MACREG( pdevice->pAU1000CtrlRegs, MAC_ADDRHI) =
		pdevice->PhysicalAddress[4] |
		pdevice->PhysicalAddress[5] << 8;

	return(0);
}


static long
MAC_AU1000_DMA_init(t_MAC_AU1000_device *pdevice)
{
	int i;

	/* The DMA registers will not be clobbered in reset. The MAC registers
		will be  This means the DMA can be setup before the MAC is brought
		out of reset but the MAC can not be */

	/*	 Setup DMA Address for Rx  */

	for (i=0; i< RX_BUFCNT ; i++) {
		MACREG(pdevice->pAU1000DMARegs, 
			(SETV(DMA_RECEIVE, MAC_DMA_TXRX) | SETV(i,MAC_DMA_BUF)  |
			SETV(DMA_ADDRESS_EN ,MAC_DMA_REG)))	|=
			pdevice->RcvBuffer[i] & MAC_DMA_ADDR_MASK;
	}

	/* Get next buffer location for DMA */

	pdevice->NextRcvBufferIndex =
		(MACREG(pdevice->pAU1000DMARegs,
		(SETV(DMA_RECEIVE, MAC_DMA_TXRX) | SETV(DMA_ADDRESS_EN,MAC_DMA_REG))) & 
		MAC_DMA_NEXTBUF_MASK) >> MAC_DMA_NEXTBUF;

	return(0);
}

/*
 * eninit():
 * Initialize the PHY and MAC.
 * This would include establishing buffer descriptor tables and
 * all the support code that will be used by the ethernet device.
 *
 * It can be assumed at this point that the array uchar BinEnetAddr[6]
 * contains the 6-byte MAC address.
 *
 * Return 0 if successful; else -1.
 */
int
eninit(void)
{
	int		i;
	ulong	tmp;

	lan_dma_buffers  = 0 ;
	MAC_AU1000_state = MAC_AU1000_DRIVER_IS_STOPPED ;

	minor_device.pAU1000CtrlRegs = (ulong*) MAC0_AU1000_BASE;
	minor_device.pAU1000EnReg = (ulong*) MAC0_AU1000_EN_BASE;
	minor_device.pAU1000DMARegs = (ulong*) MAC0_AU1000_DMA_BASE;
		
	tmp = (ulong)(minor_device.pAU1000CtrlRegs);
	tmp |= Au1500_PBUS1_OFFSET;
	minor_device.pAU1000CtrlRegs = (ulong *)tmp;

	tmp = (ulong)(minor_device.pAU1000EnReg);
	tmp |= Au1500_PBUS1_OFFSET;
	minor_device.pAU1000EnReg = (ulong *)tmp;

	/* Allocate MAC RX/TX frame buffer space */
	if (MAC_AU1000_allocate_buffers(&minor_device) != 0)
		return(-1);

	/* Stop any active DMA */
	if (MAC_AU1000_stop(&minor_device) != 0)
		return(-1);

	/* Reset MAC out of reset and program address */
	if (MAC_AU1000_control_init(&minor_device) != 0)
		return(-1);

	/* initialize AU1000 DMA context registers */
	if (MAC_AU1000_DMA_init(&minor_device) != 0)
		return(-1);

	/* initialize AU1000 MII */
	if (MAC_AU1000_MII_init(&minor_device) != 0)
		return(-1);

	/* Enable All 4 Receive DMA Here */

	for (i=0; i< RX_BUFCNT; i++) {
		
		MACREG( minor_device.pAU1000DMARegs, 
			(SETV(DMA_RECEIVE, MAC_DMA_TXRX) |			
			SETV(i,MAC_DMA_BUF)  |				/* i selects buffer to enable */
			SETV(DMA_ADDRESS_EN ,MAC_DMA_REG)) )		
			|= MAC_DMA_ENABLE_MASK;
	}

	/* Enable TX and RX */
	MACREG(minor_device.pAU1000CtrlRegs,MAC_CONTROL) |=
		SET(MAC_CONTROL_TE) | SET(MAC_CONTROL_RE);

	MAC_AU1000_state = MAC_AU1000_DRIVER_IS_STARTED;
	return(0) ;
}

/*
 * polletherdev():
 * Called continuously by the monitor (ethernet.c) to determine if there
 * is any incoming ethernet packets.
 */
int
polletherdev(void)
{
	int 				 BufferIndex;
	int					 pktcnt;
	ulong				 status ;
	ulong				 *pPacket;
	ulong				 wBufLen ; 
	t_MAC_AU1000_device *pdevice ;

	pktcnt = 0;

	/* get device context for this minor device */
	pdevice = &minor_device ;
		
	
	while (1) 
	{
		/* address next packet */

		BufferIndex = pdevice->NextRcvBufferIndex;


		if ( MACREG( pdevice->pAU1000DMARegs, 
			(SETV(DMA_RECEIVE, MAC_DMA_TXRX) |		
			SETV(BufferIndex,MAC_DMA_BUF)  | /* BufferIndex picks correct DMA register */
			SETV(DMA_ADDRESS_EN ,MAC_DMA_REG)))			
			& MAC_DMA_DONE_MASK )			 /* If DONE then a receive has taken place */
		{

			status = MACREG( pdevice->pAU1000DMARegs, 
							(SETV(DMA_RECEIVE, MAC_DMA_TXRX) |		
							SETV(BufferIndex,MAC_DMA_BUF)  | /* BufferIndex picks correct DMA register */
							SETV(DMA_STATUS ,MAC_DMA_REG)));
	 
			/* check for missed frame */
			if ( status & MAC_RECEIVE_MI_MASK )
			{
				/* received packet has	errors, drop this packet */
				pdevice->status.rx_errors++;
				if (status & MAC_RECEIVE_CR_MASK)
					 pdevice->status.rx_cr_errors++;

				if (status & MAC_RECEIVE_DB_MASK)
					 pdevice->status.rx_db_errors++;

				if (status & MAC_RECEIVE_FL_MASK)
					 pdevice->status.rx_fl_errors++;

				if (status & MAC_RECEIVE_MI_MASK)
					 pdevice->status.rx_mi_errors++;

				break;
			}
			else if( !(status & MAC_RECEIVE_PF_MASK))
			{
				/* Broadcast Frame ? */
				pdevice->status.rx_pf_errors++;
				break;
			}
			else  /* received packet is good */
			{

			   pPacket = (ulong*)KSEG0(pdevice->RcvBuffer[BufferIndex]);

			   wBufLen = status & MAC_RECEIVE_L_MASK ;
			   
			   pdevice->status.rx_bytes += wBufLen;
			   
			   EtherRFRAMECnt++;

			   /* we have a registered receive handler */
			   if ( wBufLen == 0 )
			   {
				   /* drop the packet */
				   pdevice->status.rx_zero_length_errors++;
			   }
			   else
			   {
				   /* count any broadcast's */
				   if (!memcmp(((char*)pPacket),(char *)BroadcastAddr,SYS_MAC_ADDR_SIZE))
				   {
					   pdevice->status.multicast++ ;
				   }

				   /* call the handler */
					processPACKET((struct ether_header *)pPacket,wBufLen);
					pktcnt++;

				   /* packet processed */
			   }
			}
			
			/* Enable Current buffer again since we are done */

			MACREG( pdevice->pAU1000DMARegs, 
				(SETV(DMA_RECEIVE, MAC_DMA_TXRX) |			
				SETV(BufferIndex,MAC_DMA_BUF)  |				/* i selects buffer */
				SETV(DMA_ADDRESS_EN ,MAC_DMA_REG)) )		
				= (pdevice->RcvBuffer[BufferIndex] & MAC_DMA_ADDR_MASK) | MAC_DMA_ENABLE_MASK;  /* only upper 27 bits are used */

			/* Increment Next Buffer */

			pdevice->NextRcvBufferIndex++;

			if ( pdevice->NextRcvBufferIndex >= RX_BUFCNT )
			{
				pdevice->NextRcvBufferIndex = 0;
			}
		}
		else
		{
			break;
		}
	}
	return(pktcnt);
}

/* getXmitBuffer():
 * Return a pointer to the buffer that is to be used for transmission of
 * the next packet.  Since the monitor's driver is EXTREMELY basic,
 * there will only be one packet ever being transmitted.  No need to queue
 * up transmit packets.
 */
uchar *
getXmitBuffer(void)
{
	return(tx_buff);
}

/* sendBuffer():
 * Send out the packet assumed to be built in the buffer returned by the
 * previous call to getXmitBuffer() above.
 */
int
sendBuffer(int pktlen)
{
	ulong i;
	ulong status;
	uchar *pbData = tx_buff;

	uchar						  *pbPacketData;
	ulong						  *pPacket ;
	int 						  BufferIndex;
	t_MAC_AU1000_device 		 *pdevice ;

#if INCLUDE_ETHERVERBOSE
	if (EtherVerbose &  SHOW_OUTGOING)
		printPkt((struct ether_header *)tx_buff,pktlen,ETHER_OUTGOING);
#endif

	/* get device context for this minor device */
	pdevice = &minor_device ;

	BufferIndex = (MACREG(pdevice->pAU1000DMARegs, 
									(SETV(DMA_TRANSMIT, MAC_DMA_TXRX) |		
									SETV(DMA_ADDRESS_EN,MAC_DMA_REG))) & 
									MAC_DMA_NEXTBUF_MASK) >> MAC_DMA_NEXTBUF;


	pPacket = (ulong*)KSEG1((pdevice->TxmBuffer[BufferIndex]));

	/* If either DONE or ENABLE are not 0 then DMA channel is not ready */
	/* BufferIndex picks correct DMA register... */
   if (MACREG(pdevice->pAU1000DMARegs, 
		(SETV(DMA_TRANSMIT, MAC_DMA_TXRX) |		
		SETV(BufferIndex,MAC_DMA_BUF)  |
		SETV(DMA_ADDRESS_EN ,MAC_DMA_REG))) & MAC_DMA_READY_MASK) {
		return(-1);
	}

	pbPacketData = (uchar* )pPacket ;
	
	/* copy the bits */
	memcpy((char *)pbPacketData,(char *)pbData, pktlen);
	pdevice->status.tx_bytes += pktlen;
	EtherXFRAMECnt++;
 
	/* check to PAD packet up to minimum size */
	if (pktlen < MIN_PKTSIZE) {
		memset((char *)(pbPacketData + pktlen), 0,
			(MIN_PKTSIZE-pktlen) ) ;
		pktlen = MIN_PKTSIZE ;
	}

	/* Set length register */

	MACREG( pdevice->pAU1000DMARegs, 
			(SETV(DMA_TRANSMIT, MAC_DMA_TXRX) |			 
			SETV(BufferIndex,MAC_DMA_BUF)  | 
			SETV(DMA_LENGTH,MAC_DMA_REG))) = pktlen;

	/* Enable Rx */

/*	for (i=0; i< RX_BUFCNT ; i++) {
		
		MACREG( minor_device.pAU1000DMARegs, 
			SETV(DMA_RECEIVE, MAC_DMA_TXRX) |			
			SETV(i,MAC_DMA_BUF)  |			
			SETV(DMA_ADDRESS_EN ,MAC_DMA_REG)  )		
			|= MAC_DMA_ENABLE_MASK;
	}

	pdevice->NextRcvBufferIndex = (MACREG( pdevice->pAU1000DMARegs, 
									SETV(DMA_RECEIVE, MAC_DMA_TXRX) |		
									SETV(DMA_ADDRESS_EN,MAC_DMA_REG)) & 
									MAC_DMA_NEXTBUF_MASK) >> MAC_DMA_NEXTBUF;


*/

	/* Enable DMA */
	MACREG( pdevice->pAU1000DMARegs, 
			(SETV(DMA_TRANSMIT, MAC_DMA_TXRX) |	
			SETV(BufferIndex,MAC_DMA_BUF)  |
			SETV(DMA_ADDRESS_EN ,MAC_DMA_REG)) )
			= (pdevice->TxmBuffer[BufferIndex] & MAC_DMA_ADDR_MASK) |
			  SET(MAC_DMA_ENABLE); 							



	/* Now wait until either an error occurs or transmission completes
	 * successfully.
	 */
	i = 0 ;
	while(1) {
	  /* check DMA Done bit */
	  if ( MACREG( pdevice->pAU1000DMARegs, 
			(SETV(DMA_TRANSMIT, MAC_DMA_TXRX) |			 
			SETV(BufferIndex,MAC_DMA_BUF)  | 
			SETV(DMA_ADDRESS_EN,MAC_DMA_REG))) & MAC_DMA_DONE_MASK) 
	  {
			status = MACREG( pdevice->pAU1000DMARegs, 
					(SETV(DMA_TRANSMIT, MAC_DMA_TXRX) |			 
					SETV(BufferIndex,MAC_DMA_BUF)  | 
					SETV(DMA_STATUS,MAC_DMA_REG)));

			/* Clear Done bit */

			MACREG(pdevice->pAU1000DMARegs, 
				(SETV(DMA_TRANSMIT, MAC_DMA_TXRX) |			 
				SETV(BufferIndex,MAC_DMA_BUF)  | 
				SETV(DMA_ADDRESS_EN,MAC_DMA_REG))) &= ~MAC_DMA_DONE_MASK;

			/* Check for frame abort */
			if (status & MAC_TRANSMIT_FA_MASK) {
				pdevice->status.tx_errors++;

				if (status & MAC_TRANSMIT_JT_MASK)
					pdevice->status.tx_jt_errors++;

				if (status & MAC_TRANSMIT_NC_MASK)
					pdevice->status.tx_nc_errors++;

				if (status & MAC_TRANSMIT_LS_MASK)
					pdevice->status.tx_ls_errors++;
			
				if (status & MAC_TRANSMIT_ED_MASK)
					pdevice->status.tx_ed_errors++;

				if (status & MAC_TRANSMIT_LC_MASK)
					pdevice->status.tx_lc_errors++;

				if (status & MAC_TRANSMIT_EC_MASK)
					pdevice->status.tx_lc_errors++;
				else /* Collision Count is only valid if EC is not set */
					pdevice->status.collisions += 
					((status & MAC_TRANSMIT_CC_MASK) >> MAC_TRANSMIT_CC) ;

				if (status & MAC_TRANSMIT_UR_MASK)
					pdevice->status.tx_ur_errors++;
				
			}
			break ;
		}

		/* wait 1 ms */

		// MAC_AU1000_wait( 1 ) ;

		/* wait at most 1 second */

		i++ ;
	
		/* This corresponds to 1 second at 384MHz */

		if (i > 0x16e36000) {
			pdevice->status.tx_errors++;
			pdevice->status.tx_timeout_errors++;

			/* re-init driver and controller */
			eninit();
			return(-1) ;
		  }
	}
	return(0) ;
}

void
ShowEtherdevStats(void)
{
	t_MAC_AU1000_device *pdevice;

	pdevice = &minor_device ;
	MAC_AU1000_MII_status(pdevice);
	MAC_AU1000_dump_status(pdevice);
}

void
MAC_AU1000_dump_status(t_MAC_AU1000_device *pdevice)
{

	pdevice = &minor_device;
	printf(" Bytes received:                 %ld\n\r",
		 pdevice->status.rx_bytes ) ;
	printf(" Bytes transmitted:              %ld\n\r",
		 pdevice->status.tx_bytes ) ;
	printf(" Receive errors:                 %ld\n\r",
		 pdevice->status.rx_errors ) ;
	printf(" Transmit errors:                %ld\n\r",
		 pdevice->status.tx_errors ) ;
	printf(" Multicasts received:            %ld\n\r",
		 pdevice->status.multicast ) ;
	printf(" Collisions:                     %ld\n\r",
		 pdevice->status.collisions) ;
	printf(" Receive length errors:          %ld\n\r",
		 pdevice->status.rx_length_errors ) ;
	printf(" Receive CRC errors:             %ld\n\r",
		 pdevice->status.rx_cr_errors ) ;
	printf(" Receive Dribble errors:         %ld\n\r",
		 pdevice->status.rx_db_errors ) ;
	printf(" Receive frame length:		    %ld\n\r",
		 pdevice->status.rx_fl_errors ) ;
	printf(" Transmit aborted errors:        %ld\n\r",
		 pdevice->status.tx_fa_errors) ;
	printf(" Transmit lost carrier errors:   %ld\n\r",
		 pdevice->status.tx_ls_errors) ;
	printf(" Transmit FIFO underrun errors:  %ld\n\r",
		 pdevice->status.tx_ur_errors) ;
   printf(" Transmit late collision errors: %ld\n\r",
		 pdevice->status.tx_lc_errors) ;
	printf(" Transmit timeout errors:        %ld\n\r",
		 pdevice->status.tx_ed_errors) ;
};

static
long MAC_AU1000_MII_status( t_MAC_AU1000_device *pdevice )
{
   /* check link status, spin here till done */
	while(MACREG(pdevice->pAU1000CtrlRegs,MAC_MIICTRL) &
		MAC_MIICTRL_MBUSY_MASK) ;

	/* set PHY address and read PHY Control register.
	 * (This address is 0x00 so will be read automatically
	 */
	MACREG( pdevice->pAU1000CtrlRegs, MAC_MIICTRL) =
		PHY_ADDR << MAC_MIICTRL_PHYA;

	/* check link status, spin here till done */
	while(MACREG( pdevice->pAU1000CtrlRegs, MAC_MIICTRL) &
		MAC_MIICTRL_MBUSY_MASK);

	/* check read register value */
	if (MACREG( pdevice->pAU1000CtrlRegs, MAC_MIIDATA) == PHY_CONTROL_DEFAULT)
		printf(" MII status: Link is up\n") ;
	else
		printf(" MII status: Link is down!\n" ) ;
	return(0) ;
}


/*
 * enreset():
 *	Reset the PHY and MAC.
 */
void
enreset(void)
{
	/* ADD_CODE_HERE */
}

int
EtherdevStartup(int verbose)
{
	/* Initialize local device error counts (if any) here. */

	/* Put ethernet controller in reset: */
	enreset();

	/* Initialize controller: */
	eninit();

	return(0);
}

/* disablePromiscuousReception():
 * Provide the code that disables promiscuous reception.
 */
void
disablePromiscuousReception(void)
{
	/* OPT_ADD_CODE_HERE */
}

/* enablePromiscuousReception():
 * Provide the code that enables promiscuous reception.
 */
void
enablePromiscuousReception(void)
{
	/* OPT_ADD_CODE_HERE */
}

/* disableBroadcastReception():
 * Provide the code that disables broadcast reception.
 */
void
disableBroadcastReception(void)
{
	/* ADD_CODE_HERE */
}

/* enableBroadcastReception():
 * Provide the code that enables broadcast reception.
 */
void
enableBroadcastReception(void)
{
	/* ADD_CODE_HERE */
}

/* 
 * enselftest():
 *	Run a self test of the ethernet device(s).  This can be stubbed
 *	with a return(1).
 *	Return 1 if success; else -1 if failure.
 */
int
enselftest(int verbose)
{
	return(1);
}

/* DisableEtherdev():
 * Fine as it is...
 */
void
DisableEtherdev(void)
{
	enreset();
}

/* extGetIpAdd():
 * If there was some external mechanism (other than just using the
 * IPADD shell variable established in the monrc file) for retrieval of
 * the board's IP address, then do it here...
 */
char *
extGetIpAdd(void)
{
	return((char *)0);
}

/* extGetEtherAdd():
 * If there was some external mechanism (other than just using the
 * ETHERADD shell variable established in the monrc file) for retrieval of
 * the board's MAC address, then do it here...
 */
char *
extGetEtherAdd(void)
{
	return((char *)0);
}

#endif
