/*
 * General notice:
 * This code is part of a boot-monitor package developed as a generic base
 * platform for embedded system designs.  As such, it is likely to be
 * distributed to various projects beyond the control of the original
 * author.  Please notify the author of any enhancements made or bugs found
 * so that all may benefit from the changes.  In addition, notification back
 * to the author will allow the new user to pick up changes that may have
 * been made by other users after this version of the code was distributed.
 *
 * Author:  Ed Sutter
 * email:   esutter@lucent.com
 * phone:   908-582-2351
 * 
 *
 * Modified for the CSB281 - MPC8245 Single Board
 *
 * rom_reset.s:
 */

#include "config.h"
#include "arch_ppc.h"
#include "cpu_pci.h"
#include "warmstart.h"

    .file "rom_reset.s"

    .globl  coldstart
    .globl  warmstart
    .globl  SPInit
    .extern start

    .text

#include "moncomptr.S"
#include "etheraddr.S"

    .balign     0x0100

coldstart:
    li  r3, INITIALIZE

    /* Disable I&D Cache before accessing any registers
     */
    mfspr   r5,DC_CST
    lis     r5,0x0400
    mtspr   DC_CST,r5
    isync
    mfspr   r5,IC_CST
    lis     r5,0x0400
    mtspr   IC_CST,r5
    isync

warmstart:
	xor	r0,r0,r0

    /* Initialize MSR & SRR1
     */
    lis     r5,(MSR_ME|MSR_RI)@h
    addi    r5,r5,(MSR_ME|MSR_RI)@l
    mtmsr   r5
    isync
    mtspr   srr1,r5
    isync

    /* Initialize ICTRL, ICR & DEC:
     */
    lis     r5,(ISCT_SER7)@h
    ori     r5,r5,(ISCT_SER7)@l
    mtspr   ICTRL,r5
    lis     r5,(0)@h
    ori		r5,r5,(0)@l
    mtspr   ICR,r5
    lis     r5,(0x7FFFFFFF)@h
    ori		r5,r5,(0x7FFFFFFF)@l
    mtspr   dec,r5

	li		r0,r0
	mtmsr	r0				# Disable FPU/MMU/exceptions
	isync

mem_init:
	/* initialize the cpu control registers
	 */	
	/* get the pointers to the config address and data registers */
	lis		r4, 0xfec0		/* address register */
	ori		r4, r4, 0x0000
	lis		r5, 0xfee0		/* data register */
	ori		r5, r5, 0x0000

	/* write the PCI arb and general control right away */
	/* PGCR/PACR */
	/*
	lis 	r6, 0x8000
	ori 	r6, r6, 0x0044
	lis 	r7, 0x8000
	ori 	r7, r7, 0x000f
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync
	*/

	/* FIX 06/09/2003 MJK */
	/* write the output strength to 40 ohms on everyone */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x0070
	lis 	r7, 0x0000
	ori 	r7, r7, 0x0015
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/*  setup SDRAM Bank 0 start address */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x0080
	lis 	r7, 0x0000
	ori 	r7, r7, 0x0000
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/* setup SDRAM Bank 0 end address */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x0090
	lis 	r7, 0x0000
	ori 	r7, r7, 0x003f
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/* enable SDRAM Bank 0 */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x00a0
	lis 	r7, 0x0000
	ori 	r7, r7, 0x0001
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/* setup MCCR1-4,  */
	/* MCCR1 - without MEMGO */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x00f0
	lis 	r7, 0x8800
	ori 	r7, r7, 0x0000
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/* MCCR2 */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x00f4
	lis 	r7, 0x1e00
	ori 	r7, r7, 0x023c
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/*  MCCR3 */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x00f8
	lis 	r7, 0xb600
	ori 	r7, r7, 0x0000
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/* MCCR4 */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x00fc
	lis 	r7, 0x35b0			/* was 3530 - 8/burst */
	ori 	r7, r7, 0x3334
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/* MCCR1 - with MEMGO */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x00f0
	lis 	r7, 0x8808
	ori 	r7, r7, 0x0000
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/* ERCR1 - RCS2 timing and width */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x00d0
	lis 	r7, 0xb400
	ori 	r7, r7, 0x0000
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/*  ERCR2 - RCS3 timing and width */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x00d4
	lis 	r7, 0xbcf7
	ori 	r7, r7, 0xb1e3
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/*  ERCR3 - RCS2 address and size */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x00d8
	lis 	r7, 0x7000
	ori 	r7, r7, 0x000d
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/* ERCR4 - RCS3 address and size */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x00dc
	lis 	r7, 0x7800
	ori 	r7, r7, 0x000d
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/* EUMBBAR = 0xf0000000 */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x0078
	lis 	r7, 0xf000
	ori 	r7, r7, 0x0000
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/* PICR1 */
	/* enable writes on RCS0 and RCS1 */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x00a8
	lis 	r7, 0xff14
	ori 	r7, r7, 0x1010
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

	/* PICR2 */
	/* enable 0 wait on PCI to MEM, no snoop */
	lis 	r6, 0x8000
	ori 	r6, r6, 0x00ac
	lis 	r7, 0x0800
	ori 	r7, r7, 0x0000
	stwbrx 	r6,0,r4
	sync
	stwbrx 	r7,0,r5
	sync

SPInit:
    /* initialize the stack pointer, and be assured that it is mod16 */
    lis     sp, (MonStack+MONSTACKSIZE-4)@h
    ori     sp, sp, (MonStack+MONSTACKSIZE-4)@l
    addis   r5, r0, 0xffff
    ori     r5, r5, 0xfff8
    and     r1, r1, r5

GoToC:
	ba		start
	nop
