//==========================================================================
//
// sak82c900.h
//
// Author(s):    Michael Kelly, Cogent Computer Systems, Inc.
// Contributors: 
// Date:         03/02/2003
// Description:  This file contains the register and bit defines for the
//				 Infineon SAK82C900 Dual CAN Controller
//

// Page Mode register is accessable in every page
#define SAK_PAGE		0x07c	// Page Mode Register, has address bits 11-7

//--------------------------------------------------------------------------
// Shell Registers
//
#define SAK_GCTL		0x010	// Global Control Register
#define SAK_ICTL		0x012	// Interrupt Control Register
#define SAK_CCTL		0x014	// Clock Register
#define SAK_IOM0		0x020	// I/O Mode Register 0
#define SAK_IOM2		0x022	// I/O Mode Register 2
#define SAK_IOM4		0x024	// I/O Mode Register 4
#define SAK_PIN			0x026	// PIO Input Register
#define SAK_POUT		0x028	// PIO Output Register
#define SAK_PWDN		0x040	// Power Down Register
#define SAK_IOCTL		0x042	// CAN I/O Control Register
#define SAK_CINIT		0x044	// CAN Init Register
#define SAK_CADD		0x07e	// CAN ADdress Buffer Register

//--------------------------------------------------------------------------
// CAN A Registers
//
#define SAK_A_CTL		0x200	//  Node A Control Register
#define SAK_A_STAT		0x204	//  Node A Status Register
#define SAK_A_IPEND		0x208	//  Node A Interrupt Pending Register
#define SAK_A_TMG		0x20c	//  Node A Bit Timing Register
#define SAK_A_GINP		0x210	//  Node A Global Interrupt Node Pointer Register
#define SAK_A_FCNT		0x214	//  Node A Frame Counter Register
#define SAK_A_INT0		0x218	//  Node A Interrupt ID Mask Register 0
#define SAK_A_INT4		0x21c	//  Node A Interrupt ID Mask Register 4
#define SAK_A_ECNT		0x220	//  Node A Error Counter Register

//--------------------------------------------------------------------------
// CAN A Registers
//
#define SAK_B_CTL		0x240	//  Node B Control Register
#define SAK_B_STAT		0x244	//  Node B Status Register
#define SAK_B_IPEND		0x248	//  Node B Interrupt Pending Register
#define SAK_B_TMG		0x24c	//  Node B Bit Timing Register
#define SAK_B_INP		0x250	//  Node B Global Interrupt Node Pointer Register
#define SAK_B_FCNT		0x254	//  Node B Frame Counter Register
#define SAK_B_INT0		0x258	//  Node B Interrupt ID Mask Register 0
#define SAK_B_INT4		0x25c	//  Node B Interrupt ID Mask Register 4
#define SAK_B_ERR		0x260	//  Node B Error Counter Register

//--------------------------------------------------------------------------
// Global Interrupt Pending Registers
//
#define SAK_RPEND		0x280	//  Receive Interrupt Pending Register
#define SAK_TPEND		0x2c0	//  Transmit Interrupt Register

//--------------------------------------------------------------------------
// Message Object Registers
//
// There are 32 message object blocks.  They start at 0x300 and are 0x20 apart
// Below are the definitions for the offsets within each block.
//
#define SAK_MBLK_DAT0	0x00	// Data Register, bytes 0-3
#define SAK_MBLK_DAT4	0x04	// Data Register, bytes 4-7
#define SAK_MBLK_ARB	0x08	// Arbitration Register
#define SAK_MBLK_AM		0x08	// Acceptance Mask Register
#define SAK_MBLK_CTL	0x08	// Message Control Register
#define SAK_MBLK_CFG	0x08	// Message Configuration Register
#define SAK_MBLK_FCTL	0x08	// FIFO Control Register

