/* fs.h

    Author: David de la Cruz, Technology Rescue.
    email:  ddlc@techresc.net

	This source file defines Filesystem access routines.

	Modification History :

	Version 2.1:
		02.05.2004 : new.
*/

#ifndef __fs_h_
#define	__fs_h_

#define	FILENAME_SIZE	16	// for NULL terminated string.
#define FILENAME_LEN	8	// for 8.3 filename format.
#define	FILENAME_EXT	3

/* Following structure contains media access routines pointers. */
typedef struct file_system_ops
{
	char 	name[8];		/* Name for this media. */
	ulong	start_address;	/* 32-bit address of first sector. */
	int 	(*media_read)(uchar *dst, ulong src, int size);
	int		(*media_write)(ulong dst, uchar *src, int size);

} file_system_ops;



/* Function prototypes. */

/* mount_fs function.

	Entry :
		file_system_ops * fsops : filesystem structure to initialise.

	Exit :
		file_system_ops * fsops : filesystem structure to initialise.
		return 0 if successfull, else error code.

	Function :
		It mounts filesystem structures.

*/
int mount_fs(file_system_ops * fsops);

/* fs_write function.

	Entry :
		char * filename : filename in 8.3 format.
		ulong src : 32-bit source address to copy from.
		ulong filelength : file size in bytes.

	Exit :
		<0 : error in file write.
		>0 : bytes written. (should be equal to filelength to validate write.


*/
int fs_write(char * filename, ulong src, ulong filelength);

/* fs_read function.

	Entry :
		ulong dest : destination address to copy to.
		char * filename : filename in 8.3 format.

	Exit :
		<0 : error in file read.
		>0 : bytes read.


*/
int fs_read(ulong dest, char * filename);

#endif	/* __fs_h_ */

