//==========================================================================
//
// pxa270_msc.h
//
// Author(s):    Michael Kelly, Cogent Computer Systems, Inc.
// Contributors: 
// Date:         04/17/2002
// Description:  This file contains register and bit defines for the PXA250
//               Memory Controller
//

//---------------------------------------
//  Memory Controller Register Offsets
#define MSC_MDCNFG     		0x00          			// SDRAM configuration register 0
#define MSC_MDREFR     		0x04          			// SDRAM refresh control register
#define MSC_MSC0       		0x08          			// Static memory control register 0
#define MSC_MSC1       		0x0C          			// Static memory control register 1
#define MSC_MSC2       		0x10          			// Static memory control register 2
#define MSC_MECR       		0x14          			// Expansion memory (PCMCIA / Compact Flash) bus configuration register
#define MSC_SXCNFG     		0x1C          			// Synchronous static memory control register
#define MSC_FLYCNFG    		0x20          			// Fly-by DMA DVAL<1:0> polarities
#define MSC_MCMEM0     		0x28          			// Card interface Common Memory Space Socket 0 Timing Configuration
#define MSC_MCMEM1     		0x2C          			// Card interface Common Memory Space Socket 1 Timing Configuration
#define MSC_MCATT0     		0x30          			// Card interface Attribute Space Socket 0 Timing Configuration
#define MSC_MCATT1     		0x34          			// Card interface Attribute Space Socket 1 Timing Configuration
#define MSC_MCIO0      		0x38          			// Card interface I/O Space Socket 0 Timing Configuration
#define MSC_MCIO1      		0x3C          			// Card interface I/O Space Socket 1 Timing Configuration
#define MSC_MDMRS      		0x40          			// MRS value to be written to SDRAM
#define MSC_BOOTDEF    		0x44          			// Read-Only Boot-time register. Contains BOOT_SEL and PKG_SEL values.
#define MSC_ARBCTL     		0x48          			// Arbiter Control register
#define MSC_BSCTR0     		0x4C          			// System Memory Buffer Strength Control register 0
#define MSC_BSCTR1     		0x50          			// System Memory Buffer Strength Control register 1
#define MSC_LBSCTR     		0x54          			// LCD Buffer Strength Control Register
#define MSC_MDMRSLP    		0x58          			// Special Low Power SDRAM Mode Register Set Configuration register
#define MSC_BSCTR2     		0x5C          			// System Memory Buffer Strength Control register 2
#define MSC_BSCTR3     		0x60          			// System Memory Buffer Strength Control register 3
#define MSC_SA1110     		0x64          			// SA-1110 Compatibility Mode for Static Memory register


// SDRAM configuration register 0
// Pair 2/3
#define MDCNFG_MDENX		BIT31					// SDRAM 1GB Memory Map Enable.
#define MDCNFG_DCACX2		BIT30					// Extra Column Addressing. used with MDCNFG[DCAC2].
#define MDCNFG_DSA1111_2	BIT28					// Use SA1111 Addressing Muxing Mode for pair 2/3.
#define MDCNFG_SETALWAYS2	BIT27					// Always set this bit to 1.
#define MDCNFG_DADDR2		BIT26					// Use alternate addressing for pair 2/3
#define MDCNFG_DTC2_0		(0x00 << 24)			// 00 - tRP = 2 clks, CL = 2, tRCD = 1 clks, tRAS(min) = 3 clks, tRC = 4 clks
#define MDCNFG_DTC2_1		(0x01 << 24)			// 01 - tRP = 2 clks, CL = 2, tRCD = 2 clks, tRAS(min) = 5 clks, tRC = 8 clks
#define MDCNFG_DTC2_2		(0x02 << 24)			// 10 - tRP = 3 clks, CL = 3, tRCD = 3 clks, tRAS(min) =7 clks, tRC=10 clks
#define MDCNFG_DTC2_3		(0x03 << 24)			// 11 - tRP = 3 clks, CL = 3, tRCD = 3 clks, tRAS(min) = 7 clks, tRC = 11 clks
#define MDCNFG_DNB2			BIT23					// Number of banks in partition pair 2/3, 0 = 2, 1 = 4
#define MDCNFG_DRAC2_11		(0x00 << 21)			// SDRAM row address bit count for partition pair 2/3
#define MDCNFG_DRAC2_12		(0x01 << 21)
#define MDCNFG_DRAC2_13		(0x02 << 21)
#define MDCNFG_DCAC2_8		(0x00 << 19)			// Number of Column Address bits for partition pair 2/3
#define MDCNFG_DCAC2_9		(0x01 << 19)
#define MDCNFG_DCAC2_10		(0x02 << 19)
#define MDCNFG_DCAC2_11		(0x03 << 19)
#define MDCNFG_DWID2_16		BIT18					// SDRAM data bus width for partition pair 2/3
#define MDCNFG_DE3			BIT17					// SDRAM enable for partition 3
#define MDCNFG_DE2			BIT16					// SDRAM enable for partition 2
// Pair 0/1
#define MDCNFG_STACK1		BIT15					// Determines the SDRAM address-multiplexing scheme.
#define MDCNFG_DCACX0		BIT14					// SDRAM 1GB Memory Map Enable.
#define MDCNFG_STACK0		BIT13					// Determines the SDRAM address-multiplexing scheme.
#define MDCNFG_DSA1111_0	BIT12					// Use SA1111 Addressing Muxing Mode for pair 0/1.
#define MDCNFG_SETALWAYS0	BIT11					// Always set this bit to 1.
#define MDCNFG_DADDR0		BIT10					// Use alternate addressing for pair 0/1
#define MDCNFG_DTC0_0		(0x00 << 8)				// 00 - tRP = 2 clks, CL = 2, tRCD = 1 clks, tRAS(min) = 3 clks, tRC = 4 clks
#define MDCNFG_DTC0_1		(0x01 << 8)				// 01 - tRP = 2 clks, CL = 2, tRCD = 2 clks, tRAS(min) = 5 clks, tRC = 8 clks
#define MDCNFG_DTC0_2		(0x02 << 8)				// 10 - tRP = 3 clks, CL = 3, tRCD = 3 clks, tRAS(min) =7 clks, tRC=10 clks
#define MDCNFG_DTC0_3		(0x03 << 8)				// 11 - tRP = 3 clks, CL = 3, tRCD = 3 clks, tRAS(min) = 7 clks, tRC = 11 clks
#define MDCNFG_DNB0			BIT7					// Number of banks in partition pair 0/1, 0 = 2, 1 = 4
#define MDCNFG_DRAC0_11		(0x00 << 5)				// SDRAM row address bit count for partition pair 0/1
#define MDCNFG_DRAC0_12		(0x01 << 5)
#define MDCNFG_DRAC0_13		(0x02 << 5)
#define MDCNFG_DCAC0_8		(0x00 << 3)				// Number of Column Address bits for partition pair 0/1
#define MDCNFG_DCAC0_9		(0x01 << 3)
#define MDCNFG_DCAC0_10		(0x02 << 3)
#define MDCNFG_DCAC0_11		(0x03 << 3)
#define MDCNFG_DWID0_16		BIT2					// SDRAM data bus width for partition pair 0/1, 0 = 32, 1 = 16
#define MDCNFG_DE1			BIT1					// SDRAM enable for partition 1
#define MDCNFG_DE0			BIT0					// SDRAM enable for partition 0

// SDRAM refresh control register
#define MDREFR_ALTREFA		BIT31					// 1 = SDRAM refresh is not performed after entering alternate bus master mode.
#define MDREFR_ALTREFB		BIT30					// 1 = SDRAM refresh is not performed after entering alternate bus master mode.
#define MDREFR_K0DB4		BIT29					// 1 = Divide CLK_MEM by 4 to generate SDCLK0 and SDCLK3, K0DB2 is ignored.
#define MDREFR_K2FREE		BIT25					// 1 = SDCLK2 is free-running (ignores MDREFR[APD] or MDREFR[K2RUN] bits)
#define MDREFR_K1FREE		BIT24					// 1 = SDCLK1 is free-running (ignores MDREFR[APD] or MDREFR[K1RUN] bits)
#define MDREFR_K0FREE		BIT23					// 1 = SDCLK0 is free-running (ignores MDREFR[APD] or MDREFR[K0RUN] bits)
#define MDREFR_SLFRSH		BIT22					// 1 = SDRAM Self-Refresh Enabled
#define MDREFR_APD			BIT20					// 1 = SDRAM/Synchronous Static Memory Auto-Power-Down Enable.
#define MDREFR_K2DB2		BIT19					// SDCLK2 Divide by 2, 1 = Divide by 2
#define MDREFR_K2RUN		BIT18					// SDCLK2 Run Control, 1 = Run
#define MDREFR_K1DB2		BIT17					// SDCLK1 Divide by 2, 1 = Divide by 2
#define MDREFR_K1RUN		BIT16					// SDCLK1 Run Control, 1 = Run
#define MDREFR_E1PIN		BIT15					// SDCKE1 Level Control, 1 = Enabled
#define MDREFR_K0DB2		BIT14					// SDCLK0 Divide by 2, 1 = Divide by 2
#define MDREFR_K0RUN		BIT13					// SDCLK0 Run Control, 1 = Run
#define MDREFR_DRI(_x_)		(_x_ & 0xfff)			// Refresh Interval = (Refresh time/rows) x (MEMCLK/32)

// Static memory control register 0/1/2
// Chip Selects 1/3/5
#define MSC_RBUFF1			BIT31					// Return Data Buffer vs. Streaming behavior.
#define MSC_RRR1(_x_)		((_x_ & 0x7) << 28) 	// ROM/SRAM recovery time.
#define MSC_RDN1(_x_)		((_x_ & 0xf) << 24) 	// ROM delay next access
#define MSC_RDF1(_x_)		((_x_ & 0xf) << 20) 	// ROM delay first access (0-11, 12 = 13, 13 = 15, 14 = 18, 15 = 23)
#define MSC_RBW1_16			BIT19					// ROM bus width, 0 = 32, 1 = 16
#define MSC_RT1_NBROM		(0x0 << 16)				// ROM Type, Nonburst ROM or Flash Memory
#define MSC_RT1_SRAM		(0x1 << 16)				// SRAM
#define MSC_RT1_B4ROM		(0x2 << 16)				// Burst-of-four ROM or Flash (with non-burst writes)
#define MSC_RT1_B8ROM		(0x3 << 16)				// Burst-of-eight ROM or Flash (with non-burst writes)
#define MSC_RT1_VLIO		(0x4 << 16)				// Variable Latency I/O (VLIO)
// Chip Selects 0/2/4
#define MSC_RBUFF0			BIT15					// Return Data Buffer vs. Streaming behavior.
#define MSC_RRR0(_x_)		((_x_ & 0x7) << 12) 	// ROM/SRAM recovery time.
#define MSC_RDN0(_x_)		((_x_ & 0xf) << 8) 		// ROM delay next access
#define MSC_RDF0(_x_)		((_x_ & 0xf) << 4) 		// ROM delay first access (0-11, 12 = 13, 13 = 15, 14 = 18, 15 = 23)
#define MSC_RBW0_16			BIT3					// ROM bus width, 0 = 32, 1 = 16
#define MSC_RT0_NBROM		(0x0 << 0)				// ROM Type, Nonburst ROM or Flash Memory
#define MSC_RT0_SRAM		(0x1 << 0)				// SRAM
#define MSC_RT0_B4ROM		(0x2 << 0)				// Burst-of-four ROM or Flash (with non-burst writes)
#define MSC_RT0_B8ROM		(0x3 << 0)				// Burst-of-eight ROM or Flash (with non-burst writes)
#define MSC_RT0_VLIO		(0x4 << 0)				// Variable Latency I/O (VLIO)

// Synchronous static memory control register
// Pair 2/3
#define SXCNFG_SXCLEXT2		BIT31					// Synchronous Flash Memory CAS Latency Extension for SXCL2.
#define SXCNFG_SXTP2_8		(0x2 << 28) 			// SX Memory type for pair 2/3, Sync Masked ROM
#define SXCNFG_SXTP2_16		(0x3 << 28)				// Non-SDRAM type Sync FLASH
#define SXCNFG_SXCL2_3		(0x2 << 18)				// CAS Latency for pair 2/3
#define SXCNFG_SXCL2_4		(0x3 << 18)
#define SXCNFG_SXCL2_5		(0x4 << 18)
#define SXCNFG_SXCL2_6		(0x5 << 18)
#define SXCNFG_SXCL2_7		(0x6 << 18)				// SXCNFG_SXTP2_NONSD only
#define SXCNFG_SXEN3		BIT17					// Enable Bit for Partition 3
#define SXCNFG_SXEN2		BIT16					// Enable Bit for Partition 2
// Pair 0/1
#define SXCNFG_SXCLEXT0		BIT15					// Synchronous Flash Memory CAS Latency Extension for SXCL0.
#define SXCNFG_SXTP0_8		(0x2 << 12) 			// SX Memory type for pair 0/1, burst-of-eight mode
#define SXCNFG_SXTP0_16		(0x3 << 12)				// SX Memory type for pair 0/1, burst-of-16 mode
#define SXCNFG_SXCL0_3		(0x2 << 2)				// CAS Latency for pair 0/1
#define SXCNFG_SXCL0_4		(0x3 << 2)
#define SXCNFG_SXCL0_5		(0x4 << 2)
#define SXCNFG_SXCL0_6		(0x5 << 2)
#define SXCNFG_SXCL0_7		(0x6 << 2)				// SXCNFG_SXTP2_NONSD only
#define SXCNFG_SXEN1		BIT1					// Enable Bit for Partition 1
#define SXCNFG_SXEN0		BIT0					// Enable Bit for Partition 0

// Memory Card Interface Timing Configurations
// Values are valid for MEMC0/1, MCATT0/1 and MCIO0/1
#define MCTMG_HOLD(_x_)		((_x_ & 0x3f) << 14)	// # MEMCLKs to hold address after command deassertion
#define MCTMG_ASST(_x_)		((_x_ & 0x1f) << 7)		// Code for the command assertion time. See Table 6-26.
#define MCTMG_SET(_x_)		((_x_ & 0x7f) << 0)		// # MEMCLKs to set up address before command assertion

// Expansion memory (PCMCIA / Compact Flash) bus configuration register
// MECR_CIT must be set appropriately by software after card detection 
// is complete
#define MECR_CIT			BIT1					// Card-Is-There, 0 = No card inserted, 1 = Card inserted
#define MECR_NOS			BIT0					// Number-of-Sockets, 0 = 1 Socket, 1 = 2 Sockets

// Read-Only Boot-time register. Contains BOOT_SEL and PKG_SEL values.
#define BOOTDEF_PKG_TYPE	BIT3					// 1 = 32-bit package.
#define BOOTDEF_BOOT_SEL	BIT0					// 1 = boot from 16-bit memory. 0 = boot from 32-bit memory.

// SA1110 Compatibility Mode for Static Memory.
#define SA1110_STACKN		(0x0 << 12)				// There is no stacked flash in the system.
#define SA1110_STACK0		(0x1 << 12)				// nCS0 contains stacked flash.
#define SA1110_STACK1		(0x2 << 12)				// nCS1 contains stacked flash.
#define SA1110_STACKB		(0x3 << 12)				// both nCS0 and nCS1 contain stacked flash.
#define SA1110_ENX			BIT8					// 1 = Use 2 64Mbyte chip selects (CS5:4) and 2 128Mbyte (CS1:0).
#define SA1110_5			BIT5					// 1 = Enable SA-1110 compatibility for static memory partition 5.
#define SA1110_4			BIT4					// 1 = Enable SA-1110 compatibility for static memory partition 4.
#define SA1110_3			BIT3					// 1 = Enable SA-1110 compatibility for static memory partition 3.
#define SA1110_2			BIT2					// 1 = Enable SA-1110 compatibility for static memory partition 2.
#define SA1110_1			BIT1					// 1 = Enable SA-1110 compatibility for static memory partition 1.
#define SA1110_0			BIT0					// 1 = Enable SA-1110 compatibility for static memory partition 0.
