//==========================================================================
//
// mpc5121e.h
//
// Author(s):    Michael Kelly, Cogent Computer Systems, Inc.
// Contributors: Luis Torrico, Cogent Computer Systems, Inc.
// Date:         09/18/2005
// Description:  This file contains register base addresses and offsets
//				 and access macros for the MPC5121e on-chip peripherals
//				 This was copied from the mpc5121e.h and peripherals not
//				 used by UMON have not been tested (and may not be defined).
//			     Use these defines with caution!!
//

#include "bits.h"

#ifndef ASSEMBLY_ONLY

//typedef volatile unsigned char vuint8;
//typedef volatile unsigned short vuint16;
typedef volatile unsigned long vuint32;

#endif

/* System reset offset (PowerPC standard)
 */
#define EXC_OFF_SYS_RESET		0x0100
#define	_START_OFFSET			EXC_OFF_SYS_RESET

// ------------------------------------------
// System Configuration and Memory Map - XLBMEN + MEM MAP
#define CONFIG_DEFAULT_IMMR		0xFF400000
#define DEFAULT_IMMR_REG(_x_)	*(vulong *)(CONFIG_DEFAULT_IMMR + _x_)
#define IMMR_REG(_x_)			*(vulong *)(IMMRBAR_BASE_ADD + _x_)

#define IMMRBAR			0x0000		/* Register offset to immr */
#define LPBAW			0x0020
#define LPCS0AW			0x0024
#define LPCS1AW			0x0028
#define LPCS2AW			0x002C
#define LPCS3AW			0x0030
#define LPCS4AW			0x0034
#define LPCS5AW			0x0038
#define LPCS6AW			0x003C
#define LPCA7AW			0x0040
#define SDRAMBAR		0x00A0
#define SDRAMATT		0x00A4
#define MBXBAR			0x00C0
#define SRAMBAR			0x00C4
#define NFCBAR			0x00C8

#define LPC_OFFSET		0x10000

#define CS0_CONFIG		0x00000
#define CS1_CONFIG		0x00004
#define CS2_CONFIG		0x00008
#define CS3_CONFIG		0x0000C
#define CS4_CONFIG		0x00010
#define CS5_CONFIG		0x00014
#define CS6_CONFIG		0x00018
#define CS7_CONFIG		0x0001C

#define CS_CTRL			0x00020
#define CS_CTRL_ME		0x01000000	/* CS Master Enable bit */
#define CS_CTRL_IE		0x08000000	/* CS Interrupt Enable bit */

// ------------------------------------------
// CLOCK - Clock Module
#define CLOCK_BASE_ADD		IMMRBAR_BASE_ADD + 0x00F00
#define CLOCK_REG(_x_)		*(vulong *)(CLOCK_BASE_ADD + _x_)
#define CLOCK_SPMR 			0x00 	// System PLL Mode Register
#define CLOCK_SCCR1			0x04 	// System Clock Control Register 1
#define CLOCK_SCCR2			0x08 	// System Clock Control Register 2
#define CLOCK_SCFR1			0x0C 	// System Clock Frequency Register 1
#define CLOCK_SCFR2			0x10 	// System Clock Frequency Register 2
#define CLOCK_BCR			0x18 	// Bread Crumb Register
#define CLOCK_P0CCR			0x1C 	// PSC0 Clock Control Register
#define CLOCK_P1CCR			0x20 	// PSC1 Clock Control Register
#define CLOCK_P2CCR			0x24 	// PSC2 Clock Control Register
#define CLOCK_P3CCR			0x28 	// PSC3 Clock Control Register
#define CLOCK_P4CCR			0x2C 	// PSC4 Clock Control Register
#define CLOCK_P5CCR			0x30 	// PSC5 Clock Control Register
#define CLOCK_P6CCR			0x34 	// PSC6 Clock Control Register
#define CLOCK_P7CCR			0x38 	// PSC7 Clock Control Register
#define CLOCK_P8CCR			0x3C 	// PSC8 Clock Control Register
#define CLOCK_P9CCR			0x40 	// PSC9 Clock Control Register
#define CLOCK_P10CCR		0x44 	// PSC10 Clock Control Register
#define CLOCK_P11CCR		0x48 	// PSC11 Clock Control Register
#define CLOCK_SPCCR			0x4C 	// SPDIF Clock Control Register
#define CLOCK_CCCR			0x50 	// CFM Clock Control Register
#define CLOCK_DCCR			0x54 	// DIU Clock Control Register

//---------------------------------------
// Display Interface Unit (DIU)
//
#define DIU_BASE_ADD		IMMRBAR_BASE_ADD + 0x2100
#define DIU_REG(_x_)		*(vulong *)(DIU_BASE_ADD + _x_)

#define DIU_DESC_1 			0x00 	// DIU Pointer to the Area Descriptor of Plane1
#define DIU_DESC_2			0x04 	// DIU Pointer to the Area Descriptor of Plane2
#define DIU_DESC_3			0x08 	// DIU Pointer to the Area Descriptor of Plane3
#define DIU_GAMMA			0x0C 	// DIU Pointer to Gamma Table
#define DIU_PALETTE			0x10 	// DIU Pointer to Palette
#define DIU_CURSOR			0x14 	// DIU Pointer to Cursor Bitmap
#define DIU_CURS_POS		0x18 	// DIU Position of the cursor in the display
#define DIU_MODE			0x1C 	// DIU Mode of Operation
#define DIU_BGND			0x20 	// DIU Background Color
#define DIU_BGND_WB			0x24 	// DIU Background Color in write back Mode
#define DIU_DISP_SIZE		0x28 	// DIU Display Size
#define DIU_WB_SIZE			0x2C 	// DIU Write back Plane Size
#define DIU_WB_MEM_ADDR		0x30 	// DIU Address to Store the write back Plane
#define DIU_HSYN_PARA		0x34 	// DIU Horizontal synchronization pulse parameters
#define DIU_VSYN_PARA		0x38 	// DIU Vertical synchronization pulse parameters
#define DIU_SYN_POL			0x3C 	// DIU Synchronization Signals Polarity
#define DIU_THRESHOLDS		0x40 	// DIU The Thresholds
#define DIU_INT_STATUS		0x44 	// DIU Interrupt Status Register
#define DIU_INT_MASK		0x48 	// DIU Interrupt Mask Register
#define DIU_COLORBAR_1		0x4C 	// DIU Color #1 in the Color Bar, White
#define DIU_COLORBAR_2		0x50 	// DIU Color #2 in the Color Bar, White
#define DIU_COLORBAR_3		0x54 	// DIU Color #3 in the Color Bar, White
#define DIU_COLORBAR_4		0x58 	// DIU Color #4 in the Color Bar, White
#define DIU_COLORBAR_5		0x5C 	// DIU Color #5 in the Color Bar, White
#define DIU_COLORBAR_6		0x60 	// DIU Color #6 in the Color Bar, White
#define DIU_COLORBAR_7		0x64 	// DIU Color #7 in the Color Bar, White
#define DIU_COLORBAR_8		0x68 	// DIU Color #8 in the Color Bar, White
#define DIU_FILLING			0x6C 	// DIU Input, Output Buffer Filling Status
#define DIU_PLUT			0x70 	// DIU Priority Look Up Table

// Register Bit Defines for DIU_CURS_POS
#define DIU_CURSOR_Y(_x_)	((_x_ & 0x7ff) << 16)	
#define DIU_CURSOR_X(_x_)	((_x_ & 0x7ff) << 0)	

// Register Bit Defines for DIU_DISP_SIZE
#define DIU_DELTA_Y(_x_)	((_x_ & 0x7ff) << 16)	
#define DIU_DELTA_X(_x_)	((_x_ & 0x7ff) << 0)	

// Register Bit Defines for DIU_HSYN_PARA
#define DIU_BP_H(_x_)	((_x_ & 0x3ff) << 22)	
#define DIU_PW_H(_x_)	((_x_ & 0x3ff) << 11)	
#define DIU_FP_H(_x_)	((_x_ & 0x3ff) << 0)	

// Register Bit Defines for DIU_VSYN_PARA
#define DIU_BP_V(_x_)	((_x_ & 0x3ff) << 22)	
#define DIU_PW_V(_x_)	((_x_ & 0x3ff) << 11)	
#define DIU_FP_V(_x_)	((_x_ & 0x3ff) << 0)	

// Register Bit Defines for DIU_WB_SIZE
#define DIU_WB_DELTA_Y(_x_)	((_x_ & 0x7ff) << 16)	
#define DIU_WB_DELTA_X(_x_)	((_x_ & 0x7ff) << 0)	

// Offsets for Area1
#define AREA1_PIX_FORMAT	0x00 	// Pixel Format
#define AREA1_BITMAP_ADDR	0x04 	// Bitmap Address
#define AREA1_SS_GA			0x08 	// Source size/Global alpha
#define AREA1_AOI_SIZE		0x0C 	// AOI size
#define AREA1_AOI_OFFSET	0x10 	// AOI offset
#define AREA1_DIS_OFFSET	0x14 	// Display offset
#define AREA1_CHROMA_MAX	0x18 	// Chroma key max
#define AREA1_CHROMA_MIN	0x1C 	// Chroma key min
#define AREA1_NEXT_AD		0x20 	// Next AD

// Register Bit Defines for AREA1_SS_GA
#define AREA1_SS_GA_ALPHA(_x_)	((_x_ & 0xff) << 24)	
#define AREA1_SS_GA_DELTA_YS(_x_)	((_x_ & 0x7ff) << 12)	
#define AREA1_SS_GA_DELTA_XS(_x_)	((_x_ & 0x7ff) << 0)	

// Register Bit Defines for AREA1_AOI_SIZE
#define AREA1_AOIS_FLIP(_x_)	((_x_ & 0x3) << 30)	
#define AREA1_AOIS_DELTA_YI(_x_)	((_x_ & 0x7ff) << 16)	
#define AREA1_AOIS_DELTA_XI(_x_)	((_x_ & 0x7ff) << 0)	

// Register Bit Defines for AREA1_AOI_OFFSET
#define AREA1_AOIO_DELTA_YI(_x_)	((_x_ & 0x7ff) << 16)	
#define AREA1_AOIO_DELTA_XI(_x_)	((_x_ & 0x7ff) << 0)	

// Register Bit Defines for AREA1_DIS_OFFSET
#define AREA1_DISO_OFFSET_YD(_x_)	((_x_ & 0x7ff) << 16)	
#define AREA1_DISO_OFFSET_XD(_x_)	((_x_ & 0x7ff) << 0)	

//---------------------------------------
// Programmable Serial Controller (PSC)
// additional PSC defines are in mpc5121e_psc.h
#define PSC0_BASE_ADD		IMMRBAR_BASE_ADD + 0x11000
#define PSC1_BASE_ADD		IMMRBAR_BASE_ADD + 0x11100
#define PSC2_BASE_ADD		IMMRBAR_BASE_ADD + 0x11200
#define PSC3_BASE_ADD		IMMRBAR_BASE_ADD + 0x11300
#define PSC4_BASE_ADD		IMMRBAR_BASE_ADD + 0x11400
#define PSC5_BASE_ADD		IMMRBAR_BASE_ADD + 0x11500
#define PSC6_BASE_ADD		IMMRBAR_BASE_ADD + 0x11600
#define PSC7_BASE_ADD		IMMRBAR_BASE_ADD + 0x11700
#define PSC8_BASE_ADD		IMMRBAR_BASE_ADD + 0x11800
#define PSC9_BASE_ADD		IMMRBAR_BASE_ADD + 0x11900
#define PSC10_BASE_ADD		IMMRBAR_BASE_ADD + 0x11A00
#define PSC11_BASE_ADD		IMMRBAR_BASE_ADD + 0x11B00
#define PSC0_REG_8(_x_)		*(vuchar *)(PSC0_BASE_ADD + _x_)
#define PSC0_REG_16(_x_)	*(vuint *)(PSC0_BASE_ADD + _x_)
#define PSC0_REG_32(_x_)	*(vulong *)(PSC0_BASE_ADD + _x_)
#define PSC1_REG_8(_x_)		*(vuchar *)(PSC1_BASE_ADD + _x_)
#define PSC1_REG_16(_x_)	*(vuint *)(PSC1_BASE_ADD + _x_)
#define PSC1_REG_32(_x_)	*(vulong *)(PSC1_BASE_ADD + _x_)
#define PSC2_REG_8(_x_)		*(vuchar *)(PSC2_BASE_ADD + _x_)
#define PSC2_REG_16(_x_)	*(vuint *)(PSC2_BASE_ADD + _x_)
#define PSC2_REG_32(_x_)	*(vulong *)(PSC2_BASE_ADD + _x_)
#define PSC3_REG_8(_x_)		*(vuchar *)(PSC3_BASE_ADD + _x_)
#define PSC3_REG_16(_x_)	*(vuint *)(PSC3_BASE_ADD + _x_)
#define PSC3_REG_32(_x_)	*(vulong *)(PSC3_BASE_ADD + _x_)
#define PSC4_REG_8(_x_)		*(vuchar *)(PSC4_BASE_ADD + _x_)
#define PSC4_REG_16(_x_)	*(vuint *)(PSC4_BASE_ADD + _x_)
#define PSC4_REG_32(_x_)	*(vulong *)(PSC4_BASE_ADD + _x_)
#define PSC5_REG_8(_x_)		*(vuchar *)(PSC5_BASE_ADD + _x_)
#define PSC5_REG_16(_x_)	*(vuint *)(PSC5_BASE_ADD + _x_)
#define PSC5_REG_32(_x_)	*(vulong *)(PSC5_BASE_ADD + _x_)
#define PSC6_REG_8(_x_)		*(vuchar *)(PSC6_BASE_ADD + _x_)
#define PSC6_REG_16(_x_)	*(vuint *)(PSC6_BASE_ADD + _x_)
#define PSC6_REG_32(_x_)	*(vulong *)(PSC6_BASE_ADD + _x_)
#define PSC7_REG_8(_x_)		*(vuchar *)(PSC7_BASE_ADD + _x_)
#define PSC7_REG_16(_x_)	*(vuint *)(PSC7_BASE_ADD + _x_)
#define PSC7_REG_32(_x_)	*(vulong *)(PSC7_BASE_ADD + _x_)
#define PSC8_REG_8(_x_)		*(vuchar *)(PSC8_BASE_ADD + _x_)
#define PSC8_REG_16(_x_)	*(vuint *)(PSC8_BASE_ADD + _x_)
#define PSC8_REG_32(_x_)	*(vulong *)(PSC8_BASE_ADD + _x_)
#define PSC9_REG_8(_x_)		*(vuchar *)(PSC9_BASE_ADD + _x_)
#define PSC9_REG_16(_x_)	*(vuint *)(PSC9_BASE_ADD + _x_)
#define PSC9_REG_32(_x_)	*(vulong *)(PSC9_BASE_ADD + _x_)
#define PSC10_REG_8(_x_)	*(vuchar *)(PSC10_BASE_ADD + _x_)
#define PSC10_REG_16(_x_)	*(vuint *)(PSC10_BASE_ADD + _x_)
#define PSC10_REG_32(_x_)	*(vulong *)(PSC10_BASE_ADD + _x_)
#define PSC11_REG_8(_x_)	*(vuchar *)(PSC11_BASE_ADD + _x_)
#define PSC11_REG_16(_x_)	*(vuint *)(PSC11_BASE_ADD + _x_)
#define PSC11_REG_32(_x_)	*(vulong *)(PSC11_BASE_ADD + _x_)

//---------------------------------------
// PSC Centralized FIFO Controller (FIFOC)
// additional defines are in mpc5121e_psc.h
#define FIFOC_BASE_ADD		IMMRBAR_BASE_ADD + 0xF00
#define FIFOC_REG(_x_)		*(vulong *)(FIFOC_BASE_ADD + _x_)
#define FIFO_CMD			0x00 	// Command Register
#define FIFO_ISR 			0x04 	// FIFO Interrupt Status Register

// ------------------------------------------
// LPC - LocalPlus Bus Module
#define LPC_OFFSET		0x10000
#define LPC_REG(_x_)	*(vulong *)(IMMRBAR_BASE_ADD + LPC_OFFSET + _x_)

#define CS0_CONFIG		0x00000
#define CS1_CONFIG		0x00004
#define CS2_CONFIG		0x00008
#define CS3_CONFIG		0x0000C
#define CS4_CONFIG		0x00010
#define CS5_CONFIG		0x00014
#define CS6_CONFIG		0x00018
#define CS7_CONFIG		0x0001C

#define CS_CTRL			0x00020

// ------------------------------------------
// DRAMC - DRAM Controller
#define DRAM_OFFSET		0x9000
#define DRAM_REG(_x_)	*(vulong *)(IMMRBAR_BASE_ADD + DRAM_OFFSET + _x_)

#define DDR_SYS_CONFIG 		0x00 	//  Register
#define DDR_TIME_CONFIG0	0x04 	//  Register
#define DDR_TIME_CONFIG1	0x08 	//  Register
#define DDR_TIME_CONFIG2	0x0C 	//  Register
#define DDR_COMMAND			0x10 	//  Register
#define DDR_COMP_COMMAND	0x14 	//  Register
#define DDR_SELF_REF0		0x18 	//  Register
#define DDR_SELF_REF1		0x1C 	//  Register
#define DDR_SELF_REF2		0x20 	//  Register
#define DDR_SELF_REF3		0x24 	//  Register
#define DDR_SELF_REF4		0x28 	//  Register
#define DDR_SELF_REF5		0x2C 	//  Register
#define DDR_SELF_REF6		0x30 	//  Register
#define DDR_SELF_REF7		0x34 	//  Register
#define DQS_CONF_COUNT		0x38 	//  Register
#define DQS_CONF_TIME		0x3C 	//  Register
#define DQS_DELAY_STAT		0x40 	//  Register

/* SWCRR - System Watchdog Control Register
 */
#define SWCRR				0x0904		/* Register offset to immr */
#define SWCRR_SWTC			0xFFFF0000	/* Software Watchdog Time Count */
#define SWCRR_SWEN			0x00000004	/* Watchdog Enable bit */
#define SWCRR_SWRI			0x00000002	/* Software Watchdog Reset/Interrupt Select bit */
#define SWCRR_SWPR			0x00000001	/* Software Watchdog Counter Prescale bit */
#define SWCRR_RES			~(SWCRR_SWTC | SWCRR_SWEN | SWCRR_SWRI | SWCRR_SWPR)

/* SWCNR - System Watchdog Counter Register
 */
#define SWCNR				0x0908		/* Register offset to immr */
#define SWCNR_SWCN			0x0000FFFF	/* Software Watchdog Count mask */
#define SWCNR_RES			~(SWCNR_SWCN)

/* SWSRR - System Watchdog Service Register
 */
#define SWSRR				0x090E		/* Register offset to immr */

/* GPIOC - GPIO Controller
 */
#define GPIO_OFFSET		0x1100
#define GPIO_REG(_x_)	*(vulong *)(IMMRBAR_BASE_ADD + GPIO_OFFSET + _x_)

#define GPIO_DIR	0x00 	//  Register
#define GPIO_OD		0x04 	//  Register
#define GPIO_DATA	0x08 	//  Register
#define GPIO_IER	0x0C 	//  Register
#define GPIO_IMR	0x10 	//  Register
#define GPIO_ICR1	0x14 	//  Register
#define GPIO_ICR2	0x18 	//  Register

/* IO Control Register
 */
#define IO_CTRL_OFFSET		0xA000
#define IO_CTRL_REG(_x_)	*(vulong *)(IMMRBAR_BASE_ADD + IO_CTRL_OFFSET + _x_)

#define IOCTL_MEM		0x00	// MEM Pad Control Register0	
#define IOCTL_GP		0x004
#define IOCTL_LPC_CLK		0x008
#define IOCTL_LPC_OE		0x00C
#define IOCTL_LPC_RWB		0x010
#define IOCTL_LPC_ACK		0x014
#define IOCTL_LPC_CS0		0x018
#define IOCTL_NFC_CE0		0x01C
#define IOCTL_LPC_CS1		0x020
#define IOCTL_LPC_CS2		0x024
#define IOCTL_LPC_AX03		0x028
#define IOCTL_EMB_AX02		0x02C
#define IOCTL_EMB_AX01		0x030
#define IOCTL_EMB_AX00		0x034
#define IOCTL_EMB_AD31		0x038
#define IOCTL_EMB_AD30		0x03C
#define IOCTL_EMB_AD29		0x040
#define IOCTL_EMB_AD28		0x044
#define IOCTL_EMB_AD27		0x048
#define IOCTL_EMB_AD26		0x04C
#define IOCTL_EMB_AD25		0x050
#define IOCTL_EMB_AD24		0x054
#define IOCTL_EMB_AD23		0x058
#define IOCTL_EMB_AD22		0x05C
#define IOCTL_EMB_AD21		0x060
#define IOCTL_EMB_AD20		0x064
#define IOCTL_EMB_AD19		0x068
#define IOCTL_EMB_AD18		0x06C
#define IOCTL_EMB_AD17		0x070
#define IOCTL_EMB_AD16		0x074
#define IOCTL_EMB_AD15		0x078
#define IOCTL_EMB_AD14		0x07C
#define IOCTL_EMB_AD13		0x080
#define IOCTL_EMB_AD12		0x084
#define IOCTL_EMB_AD11		0x088
#define IOCTL_EMB_AD10		0x08C
#define IOCTL_EMB_AD09		0x090
#define IOCTL_EMB_AD08		0x094
#define IOCTL_EMB_AD07		0x098
#define IOCTL_EMB_AD06		0x09C
#define IOCTL_EMB_AD05		0x0A0
#define IOCTL_EMB_AD04		0x0A4
#define IOCTL_EMB_AD03		0x0A8
#define IOCTL_EMB_AD02		0x0AC
#define IOCTL_EMB_AD01		0x0B0
#define IOCTL_EMB_AD00		0x0B4
#define IOCTL_PATA_CE1		0x0B8
#define IOCTL_PATA_CE2		0x0BC
#define IOCTL_PATA_ISOLATE	0x0C0
#define IOCTL_PATA_IOR		0x0C4
#define IOCTL_PATA_IOW		0x0C8
#define IOCTL_PATA_IOCHRDY	0x0CC
#define IOCTL_PATA_INTRQ	0x0D0
#define IOCTL_PATA_DRQ		0x0D4
#define IOCTL_PATA_DACK		0x0D8
#define IOCTL_NFC_WP		0x0DC
#define IOCTL_NFC_RB		0x0E0
#define IOCTL_NFC_ALE		0x0E4
#define IOCTL_NFC_CLE		0x0E8
#define IOCTL_NFC_WE		0x0EC
#define IOCTL_NFC_RE		0x0F0
#define IOCTL_PCI_AD31		0x0F4
#define IOCTL_PCI_AD30		0x0F8
#define IOCTL_PCI_AD29		0x0FC
#define IOCTL_PCI_AD28		0x100
#define IOCTL_PCI_AD27		0x104
#define IOCTL_PCI_AD26		0x108
#define IOCTL_PCI_AD25		0x10C
#define IOCTL_PCI_AD24		0x110
#define IOCTL_PCI_AD23		0x114
#define IOCTL_PCI_AD22		0x118
#define IOCTL_PCI_AD21		0x11C
#define IOCTL_PCI_AD20		0x120
#define IOCTL_PCI_AD19		0x124
#define IOCTL_PCI_AD18		0x128
#define IOCTL_PCI_AD17		0x12C
#define IOCTL_PCI_AD16		0x130
#define IOCTL_PCI_AD15		0x134
#define IOCTL_PCI_AD14		0x138
#define IOCTL_PCI_AD13		0x13C
#define IOCTL_PCI_AD12		0x140
#define IOCTL_PCI_AD11		0x144
#define IOCTL_PCI_AD10		0x148
#define IOCTL_PCI_AD09		0x14C
#define IOCTL_PCI_AD08		0x150
#define IOCTL_PCI_AD07		0x154
#define IOCTL_PCI_AD06		0x158
#define IOCTL_PCI_AD05		0x15C
#define IOCTL_PCI_AD04		0x160
#define IOCTL_PCI_AD03		0x164
#define IOCTL_PCI_AD02		0x168
#define IOCTL_PCI_AD01		0x16C
#define IOCTL_PCI_AD00		0x170
#define IOCTL_PCI_CBE0		0x174
#define IOCTL_PCI_CBE1		0x178
#define IOCTL_PCI_CBE2		0x17C
#define IOCTL_PCI_CBE3		0x180
#define IOCTL_PCI_GNT2		0x184
#define IOCTL_PCI_REQ2		0x188
#define IOCTL_PCI_GNT1		0x18C
#define IOCTL_PCI_REQ1		0x190
#define IOCTL_PCI_GNT0		0x194
#define IOCTL_PCI_REQ0		0x198
#define IOCTL_PCI_INTA		0x19C
#define IOCTL_PCI_CLK		0x1A0
#define IOCTL_PCI_RST_OUT	0x1A4
#define IOCTL_PCI_FRAME		0x1A8
#define IOCTL_PCI_IDSEL		0x1AC
#define IOCTL_PCI_DEVSEL	0x1B0
#define IOCTL_PCI_IRDY		0x1B4
#define IOCTL_PCI_TRDY		0x1B8
#define IOCTL_PCI_STOP		0x1BC
#define IOCTL_PCI_PAR		0x1C0
#define IOCTL_PCI_PERR		0x1C4
#define IOCTL_PCI_SERR		0x1C8
#define IOCTL_SPDIF_TXCLK	0x1CC
#define IOCTL_SPDIF_TX		0x1D0
#define IOCTL_SPDIF_RX		0x1D4
#define IOCTL_I2C0_SCL		0x1D8
#define IOCTL_I2C0_SDA		0x1DC
#define IOCTL_I2C1_SCL		0x1E0
#define IOCTL_I2C1_SDA		0x1E4
#define IOCTL_I2C2_SCL		0x1E8
#define IOCTL_I2C2_SDA		0x1EC
#define IOCTL_IRQ0		0x1F0
#define IOCTL_IRQ1		0x1F4
#define IOCTL_CAN1_TX		0x1F8
#define IOCTL_CAN2_TX		0x1FC
#define IOCTL_J1850_TX		0x200
#define IOCTL_J1850_RX		0x204
#define IOCTL_PSC_MCLK_IN	0x208
#define IOCTL_PSC0_0		0x20C
#define IOCTL_PSC0_1		0x210
#define IOCTL_PSC0_2		0x214
#define IOCTL_PSC0_3		0x218
#define IOCTL_PSC0_4		0x21C
#define IOCTL_PSC1_0		0x220
#define IOCTL_PSC1_1		0x224
#define IOCTL_PSC1_2		0x228
#define IOCTL_PSC1_3		0x22C
#define IOCTL_PSC1_4		0x230
#define IOCTL_PSC2_0		0x234
#define IOCTL_PSC2_1		0x238
#define IOCTL_PSC2_2		0x23C
#define IOCTL_PSC2_3		0x240
#define IOCTL_PSC2_4		0x244
#define IOCTL_PSC3_0		0x248
#define IOCTL_PSC3_1		0x24C
#define IOCTL_PSC3_2		0x250
#define IOCTL_PSC3_3		0x254
#define IOCTL_PSC3_4		0x258
#define IOCTL_PSC4_0		0x25C
#define IOCTL_PSC4_1		0x260
#define IOCTL_PSC4_2		0x264
#define IOCTL_PSC4_3		0x268
#define IOCTL_PSC4_4		0x26C
#define IOCTL_PSC5_0		0x270
#define IOCTL_PSC5_1		0x274
#define IOCTL_PSC5_2		0x278
#define IOCTL_PSC5_3		0x27C
#define IOCTL_PSC5_4		0x280
#define IOCTL_PSC6_0		0x284
#define IOCTL_PSC6_1		0x288
#define IOCTL_PSC6_2		0x28C
#define IOCTL_PSC6_3		0x290
#define IOCTL_PSC6_4		0x294
#define IOCTL_PSC7_0		0x298
#define IOCTL_PSC7_1		0x29C
#define IOCTL_PSC7_2		0x2A0
#define IOCTL_PSC7_3		0x2A4
#define IOCTL_PSC7_4		0x2A8
#define IOCTL_PSC8_0		0x2AC
#define IOCTL_PSC8_1		0x2B0
#define IOCTL_PSC8_2		0x2B4
#define IOCTL_PSC8_3		0x2B8
#define IOCTL_PSC8_4		0x2BC
#define IOCTL_PSC9_0		0x2C0
#define IOCTL_PSC9_1		0x2C4
#define IOCTL_PSC9_2		0x2C8
#define IOCTL_PSC9_3		0x2CC
#define IOCTL_PSC9_4		0x2D0
#define IOCTL_PSC10_0		0x2D4
#define IOCTL_PSC10_1		0x2D8
#define IOCTL_PSC10_2		0x2DC
#define IOCTL_PSC10_3		0x2E0
#define IOCTL_PSC10_4		0x2E4
#define IOCTL_PSC11_0		0x2E8
#define IOCTL_PSC11_1		0x2EC
#define IOCTL_PSC11_2		0x2F0
#define IOCTL_PSC11_3		0x2F4
#define IOCTL_PSC11_4		0x2F8
#define IOCTL_HRESET		0x2FC
#define IOCTL_SRESET		0x300
#define IOCTL_CKSTP_OUT		0x304
#define IOCTL_USB2_VBUS_PWR_FAULT	0x308
#define IOCTL_USB2_VBUS_PWR_SELECT	0x30C
#define IOCTL_USB2_PHY_DRVV_BUS		0x310

/*********************************************************************
*
* Fast Ethernet Controller (FEC)
*
*********************************************************************/
 /* Register Offset Base */
#define MPC512X_FECL_OFFSET		0x2800
#define MPC512X_FEC_REG(_x_)	*(vulong *)(IMMRBAR_BASE_ADD + MPC512X_FECL_OFFSET + _x_)


/* Register read/write macros */
#define FEC_EIR                   (*(vuint32*)(0x80002804))
#define FEC_EIMR                  (*(vuint32*)(0x80002808))
#define FEC_RDAR                  (*(vuint32*)(0x80002810))
#define FEC_TDAR                  (*(vuint32*)(0x80002814))
#define FEC_ECR                   (*(vuint32*)(0x80002824))
#define FEC_MMFR                  (*(vuint32*)(0x80002840))
#define FEC_MSCR                  (*(vuint32*)(0x80002844))
#define FEC_MIBC                  (*(vuint32*)(0x80002864))
#define FEC_RCR                   (*(vuint32*)(0x80002884))
#define FEC_TCR                   (*(vuint32*)(0x800028C4))
#define FEC_PALR                  (*(vuint32*)(0x800028E4))
#define FEC_PAUR                  (*(vuint32*)(0x800028E8))
#define FEC_OPD                   (*(vuint32*)(0x800028EC))
#define FEC_IAUR                  (*(vuint32*)(0x80002918))
#define FEC_IALR                  (*(vuint32*)(0x8000291C))
#define FEC_GAUR                  (*(vuint32*)(0x80002920))
#define FEC_GALR                  (*(vuint32*)(0x80002924))
#define FEC_TFWR                  (*(vuint32*)(0x80002944))
#define FEC_FRBR                  (*(vuint32*)(0x8000294C))
#define FEC_FRSR                  (*(vuint32*)(0x80002950))
#define FEC_ERDSR                 (*(vuint32*)(0x80002980))
#define FEC_ETDSR                 (*(vuint32*)(0x80002984))
#define FEC_EMRBR                 (*(vuint32*)(0x80002988))
#define FEC_DMACTRL               (*(vuint32*)(0x800029F4))
#define FEC_RMON_T_DROP           (*(vuint32*)(0x80002A00))
#define FEC_RMON_T_PACKETS        (*(vuint32*)(0x80002A04))
#define FEC_RMON_T_BC_PKT         (*(vuint32*)(0x80002A08))
#define FEC_RMON_T_MC_PKT         (*(vuint32*)(0x80002A0C))
#define FEC_RMON_T_CRC_ALIGN      (*(vuint32*)(0x80002A10))
#define FEC_RMON_T_UNDERSIZE      (*(vuint32*)(0x80002A14))
#define FEC_RMON_T_OVERSIZE       (*(vuint32*)(0x80002A18))
#define FEC_RMON_T_FRAG           (*(vuint32*)(0x80002A1C))
#define FEC_RMON_T_JAB            (*(vuint32*)(0x80002A20))
#define FEC_RMON_T_COL            (*(vuint32*)(0x80002A24))
#define FEC_RMON_T_P64            (*(vuint32*)(0x80002A28))
#define FEC_RMON_T_P65TO127       (*(vuint32*)(0x80002A2C))
#define FEC_RMON_T_P128TO255      (*(vuint32*)(0x80002A30))
#define FEC_RMON_T_P256TO511      (*(vuint32*)(0x80002A34))
#define FEC_RMON_T_P512TO1023     (*(vuint32*)(0x80002A38))
#define FEC_RMON_T_P1024TO2047    (*(vuint32*)(0x80002A3C))
#define FEC_RMON_T_P_GTE2048      (*(vuint32*)(0x80002A40))
#define FEC_RMON_T_OCTETS         (*(vuint32*)(0x80002A44))
#define FEC_IEEE_T_DROP           (*(vuint32*)(0x80002A48))
#define FEC_IEEE_T_FRAME_OK       (*(vuint32*)(0x80002A4C))
#define FEC_IEEE_T_1COL           (*(vuint32*)(0x80002A50))
#define FEC_IEEE_T_MCOL           (*(vuint32*)(0x80002A54))
#define FEC_IEEE_T_DEF            (*(vuint32*)(0x80002A58))
#define FEC_IEEE_T_LCOL           (*(vuint32*)(0x80002A5C))
#define FEC_IEEE_T_EXCOL          (*(vuint32*)(0x80002A60))
#define FEC_IEEE_T_MACERR         (*(vuint32*)(0x80002A64))
#define FEC_IEEE_T_CSERR          (*(vuint32*)(0x80002A68))
#define FEC_IEEE_T_SQE            (*(vuint32*)(0x80002A6C))
#define FEC_IEEE_T_FDXFC          (*(vuint32*)(0x80002A70))
#define FEC_IEEE_T_OCTETS_OK      (*(vuint32*)(0x80002A74))
#define FEC_RMON_R_PACKETS        (*(vuint32*)(0x80002A84))
#define FEC_RMON_R_BC_PKT         (*(vuint32*)(0x80002A88))
#define FEC_RMON_R_MC_PKT         (*(vuint32*)(0x80002A8C))
#define FEC_RMON_R_CRC_ALIGN      (*(vuint32*)(0x80002A90))
#define FEC_RMON_R_UNDERSIZE      (*(vuint32*)(0x80002A94))
#define FEC_RMON_R_OVERSIZE       (*(vuint32*)(0x80002A98))
#define FEC_RMON_R_FRAG           (*(vuint32*)(0x80002A9C))
#define FEC_RMON_R_JAB            (*(vuint32*)(0x80002AA0))
#define FEC_RMON_R_RESVD_0        (*(vuint32*)(0x80002AA4))
#define FEC_RMON_R_P64            (*(vuint32*)(0x80002AA8))
#define FEC_RMON_R_P65TO127       (*(vuint32*)(0x80002AAC))
#define FEC_RMON_R_P128TO255      (*(vuint32*)(0x80002AB0))
#define FEC_RMON_R_P256TO511      (*(vuint32*)(0x80002AB4))
#define FEC_RMON_R_512TO1023      (*(vuint32*)(0x80002AB8))
#define FEC_RMON_R_P_GTE2048      (*(vuint32*)(0x80002AC0))
#define FEC_RMON_R_1024TO2047     (*(vuint32*)(0x80002ABC))
#define FEC_RMON_R_OCTETS         (*(vuint32*)(0x80002AC4))
#define FEC_IEEE_R_DROP           (*(vuint32*)(0x80002AC8))
#define FEC_IEEE_R_FRAME_OK       (*(vuint32*)(0x80002ACC))
#define FEC_IEEE_R_CRC            (*(vuint32*)(0x80002AD0))
#define FEC_IEEE_R_ALIGN          (*(vuint32*)(0x80002AD4))
#define FEC_IEEE_R_MACERR         (*(vuint32*)(0x80002AD8))
#define FEC_IEEE_R_FDXFC          (*(vuint32*)(0x80002ADC))
#define FEC_IEEE_R_OCTETS_OK      (*(vuint32*)(0x80002AE0))


/* Bit definitions and macros for FEC_EIR */
#define FEC_EIR_UN                (0x00080000)
#define FEC_EIR_RL                (0x00100000)
#define FEC_EIR_LC                (0x00200000)
#define FEC_EIR_EBERR             (0x00400000)
#define FEC_EIR_MII               (0x00800000)
#define FEC_EIR_RXB               (0x01000000)
#define FEC_EIR_RXF               (0x02000000)
#define FEC_EIR_TXB               (0x04000000)
#define FEC_EIR_TXF               (0x08000000)
#define FEC_EIR_GRA               (0x10000000)
#define FEC_EIR_BABT              (0x20000000)
#define FEC_EIR_BABR              (0x40000000)
#define FEC_EIR_HBERR             (0x80000000)
#define FEC_EIR_CLEAR_ALL         (0xFFFFFFFF)

/* Bit definitions and macros for FEC_EIMR */
#define FEC_EIMR_UN               (0x00080000)
#define FEC_EIMR_RL               (0x00100000)
#define FEC_EIMR_LC               (0x00200000)
#define FEC_EIMR_EBERR            (0x00400000)
#define FEC_EIMR_MII              (0x00800000)
#define FEC_EIMR_RXB              (0x01000000)
#define FEC_EIMR_RXF              (0x02000000)
#define FEC_EIMR_TXB              (0x04000000)
#define FEC_EIMR_TXF              (0x08000000)
#define FEC_EIMR_GRA              (0x10000000)
#define FEC_EIMR_BABT             (0x20000000)
#define FEC_EIMR_BABR             (0x40000000)
#define FEC_EIMR_HBERR            (0x80000000)
#define FEC_EIMR_MASK_ALL         (0x00000000)
#define FEC_EIMR_UNMASK_ALL       (0xFFFFFFFF)

/* Bit definitions and macros for FEC_RDAR */
#define FEC_RDAR_R_DES_ACTIVE     (0x01000000)

/* Bit definitions and macros for FEC_TDAR */
#define FEC_TDAR_X_DES_ACTIVE     (0x01000000)

/* Bit definitions and macros for FEC_ECR */
#define FEC_ECR_RESET             (0x00000001)
#define FEC_ECR_ETHER_EN          (0x00000002)

/* Bit definitions and macros for FEC_MMFR */
#define FEC_MMFR_DATA(x)          (((x)&0x0000FFFF)<<0)
#define FEC_MMFR_TA(x)            (((x)&0x00000003)<<16)
#define FEC_MMFR_RA(x)            (((x)&0x0000001F)<<18)
#define FEC_MMFR_PA(x)            (((x)&0x0000001F)<<23)
#define FEC_MMFR_OP(x)            (((x)&0x00000003)<<28)
#define FEC_MMFR_ST(x)            (((x)&0x00000003)<<30)
#define FEC_MMFR_ST_01            (0x40000000)
#define FEC_MMFR_OP_READ          (0x20000000)
#define FEC_MMFR_OP_WRITE         (0x10000000)
#define FEC_MMFR_TA_10            (0x00020000)

/* Bit definitions and macros for FEC_MSCR */
#define FEC_MSCR_MII_SPEED(x)     (((x)&0x0000003F)<<1)
#define FEC_MSCR_DIS_PREAMBLE     (0x00000080)

/* Bit definitions and macros for FEC_MIBC */
#define FEC_MIBC_MIB_IDLE         (0x40000000)
#define FEC_MIBC_MIB_DISABLE      (0x80000000)

/* Bit definitions and macros for FEC_RCR */
#define FEC_RCR_LOOP              (0x00000001)
#define FEC_RCR_DRT               (0x00000002)
#define FEC_RCR_MII_MODE          (0x00000004)
#define FEC_RCR_PROM              (0x00000008)
#define FEC_RCR_BC_REJ            (0x00000010)
#define FEC_RCR_FCE               (0x00000020)
#define FEC_RCR_MAX_FL(x)         (((x)&0x000007FF)<<16)

/* Bit definitions and macros for FEC_TCR */
#define FEC_TCR_GTS               (0x00000001)
#define FEC_TCR_HBC               (0x00000002)
#define FEC_TCR_FDEN              (0x00000004)
#define FEC_TCR_TFC_PAUSE         (0x00000008)
#define FEC_TCR_RFC_PAUSE         (0x00000010)

/* Bit definitions and macros for FEC_PALR */
#define FEC_PALR_PADDR1(x)        (((x)&0xFFFFFFFF)<<0)

/* Bit definitions and macros for FEC_PAUR */
#define FEC_PAUR_TYPE(x)          (((x)&0x0000FFFF)<<0)
#define FEC_PAUR_PADDR2(x)        (((x)&0xFFFF)<<16)

/* Bit definitions and macros for FEC_OPD */
#define FEC_OPD_PAUSE_DUR(x)      (((x)&0x0000FFFF)<<0)
#define FEC_OPD_OPCODE(x)         (((x)&0x0000FFFF)<<16)

/* Bit definitions and macros for FEC_IAUR */
#define FEC_IAUR_IADDR1(x)        (((x)&0xFFFFFFFF)<<0)

/* Bit definitions and macros for FEC_IALR */
#define FEC_IALR_IADDR2(x)        (((x)&0xFFFFFFFF)<<0)

/* Bit definitions and macros for FEC_GAUR */
#define FEC_GAUR_GADDR1(x)        (((x)&0xFFFFFFFF)<<0)

/* Bit definitions and macros for FEC_GALR */
#define FEC_GALR_GADDR2(x)        (((x)&0xFFFFFFFF)<<0)

/* Bit definitions and macros for FEC_TFWR */
#define FEC_TFWR_X_WMRK(x)        (((x)&0x00000003)<<0)

/* Bit definitions and macros for FEC_FRBR */
#define FEC_FRBR_R_BOUND(x)       (((x)&0x000000FF)<<2)

/* Bit definitions and macros for FEC_FRSR */
#define FEC_FRSR_R_FSTART(x)      (((x)&0x000000FF)<<2)

/* Bit definitions and macros for FEC_ERDSR */
#define FEC_ERDSR_R_DES_START(x)  (((x)&0x3FFFFFFF)<<2)

/* Bit definitions and macros for FEC_ETDSR */
#define FEC_ETDSR_X_DES_START(x)  (((x)&0x3FFFFFFF)<<2)

/* Bit definitions and macros for FEC_EMRBR */
#define FEC_EMRBR_R_BUF_SIZE(x)   (((x)&0x0000007F)<<4)

#define FEC_TxBD_R					0x8000
#define FEC_TxBD_BUSY				0x4000
#define FEC_TxBD_TO1				0x4000
#define FEC_TxBD_W					0x2000
#define FEC_TxBD_TO2				0x1000
#define FEC_TxBD_FIRST				0x1000
#define FEC_TxBD_L					0x0800
#define FEC_TxBD_TC					0x0400
#define FEC_TxBD_DEF				0x0200
#define FEC_TxBD_HB					0x0100
#define FEC_TxBD_LC					0x0080
#define FEC_TxBD_RL					0x0040
#define FEC_TxBD_UN					0x0002
#define FEC_TxBD_CSL				0x0001
#define FEC_RxBD_E					0x8000
#define FEC_RxBD_INUSE				0x4000
#define FEC_RxBD_R01				0x4000
#define FEC_RxBD_W					0x2000
#define FEC_RxBD_R02				0x1000
#define FEC_RxBD_L					0x0800
#define FEC_RxBD_M					0x0100
#define FEC_RxBD_BC					0x0080
#define FEC_RxBD_MC					0x0040
#define FEC_RxBD_LG					0x0020
#define FEC_RxBD_NO					0x0010
#define FEC_RxBD_CR					0x0004
#define FEC_RxBD_OV					0x0002
#define FEC_RxBD_TR					0x0001

