//==========================================================================
//
// mpc5121e_psc.h
//
// Author(s):    Luis Torrico, Cogent Computer Systems, Inc.
// Contributors: 
// Date:         10/06/2008
// Description:  This file contains offsets and bit defines
//				 for the MPC5121e on-chip PSC's
//

#include "bits.h"

// Register offsets
#define PSC_MR1			0x00		// Mode Register 1
#define PSC_MR2			0x00		// Mode Register 2
#define PSC_SR			0x04		// Status Register 				
#define PSC_CSR			0x04		// Clock Select Register 				
#define PSC_CR			0x08		// Command Register 				
#define PSC_RXB			0x0C		// Rx Buffer Register 				
#define PSC_TXB			0x0C		// Tx Buffer Register 			
#define PSC_IPCR		0x10		// Input Port Change Register 				
#define PSC_ACR			0x10		// Auxiliary Control Register 				
#define PSC_ISR			0x14		// Interrupt Status Register 		
#define PSC_IMR			0x14		// Interrupt Mask Register 			
#define PSC_CTUR		0x18		// Counter Timer Upper Register			
#define PSC_CTLR		0x1C		// Counter Timer Lower Register 			
#define PSC_CCR			0x20		// Codec Clock Register 			
#define PSC_AC97_SLOT	0x24		// AC97 Slots Register 
#define PSC_AC97_CMD	0x28		// AC97 Command register
#define PSC_AC97_DATA	0x2C		// AC97 Status Data Register 
#define PSC_IVR			0x30		// Interrupt Vector Register 
#define PSC_IP			0x34		// Input Port Register 	
#define PSC_OP1			0x38		// Output Port 1 Bit Set Register 	
#define PSC_OP0			0x3C		// Output Port 0 Bit Set Register
#define PSC_SICR		0x40		// Serial Interface Control Register 	

// Bit defines - Refer to MPC5121e Users Manual for more detailed information
//
// PSC_MR1 - Mode Register 1 : UART mode only 				
#define PSC_RX_RTS		BIT7		// 1 = RTS is negated if the PSC FIFO is full
#define PSC_TO_EN		BIT5		// 1 = Time out counter is enabled
#define PSC_PM_WP		(0 << 3)	// Parity Mode = With Parity
#define PSC_PM_FP		(1 << 3)	// Parity Mode = Force parity
#define PSC_PM_NP		(2 << 3)	// Parity Mode = No Parity
#define PSC_PM_MM		(3 << 3)	// Parity Mode = Multidrop mode
#define PSC_PT			BIT2		// Parity Type
#define PSC_B_C_5		(0 << 0)	// Selects 5 bits per character to be sent
#define PSC_B_C_6		(1 << 0)	// Selects 6 bits per character to be sent
#define PSC_B_C_7		(2 << 0)	// Selects 7 bits per character to be sent
#define PSC_B_C_8		(3 << 0)	// Selects 8 bits per character to be sent

// PSC_MR2 - Mode Register 2 				
#define PSC_CM_NORM		(0 << 6)			// Channel Mode is Normal
#define PSC_CM_AUTO		(1 << 6)			// Channel Mode is Automatic Echo
#define PSC_CM_LLB		(2 << 6)			// Channel Mode is Local Loop-back
#define PSC_CM_RLB		(3 << 6)			// Channel Mode is Remote Loop-back
#define PSC_TX_RTS		BIT5				// Transmitter ready-to-send, UART mode only
#define PSC_TX_CTS		BIT4				// Transmitter clear-to-send, UART mode only
#define PSC_SB(_x_)		((_x_ & 0xf) << 0)	// Stop bit length control
#define PSC_RX_1		(0 << 2)			// 1 RX channel, Codec mode only
#define PSC_RX_2		(1 << 2)			// 2 RX channel, Codec mode only
#define PSC_RX_3		(2 << 2)			// 3 RX channel, Codec mode only
#define PSC_TX_1		(0 << 0)			// 1 TX channel, Codec mode only
#define PSC_TX_2		(1 << 0)			// 2 TX channel, Codec mode only
#define PSC_TX_3		(2 << 0)			// 3 TX channel, Codec mode only

// PSC_SR - Status Register : Read-only for UART mode				
//#define PSC_USR_RB			BIT15		// 1 = Received Break
//#define PSC_USR_FE			BIT14		// 1 = Framing error detected
//#define PSC_USR_PE			BIT13		// 1 = Parity error	detected
//#define PSC_USR_ORERR		BIT12		// 1 = Overrun error detected
//#define PSC_USR_TXEMP		BIT11		// 1 = Transmitter empty
//#define PSC_USR_CDE			BIT7		// 1 = No DCD error
//#define PSC_USR_ERROR		BIT6		// 1 = Error status detected
//#define PSC_USR_TIMEOUT		BIT5		// 1 = Time out event detected

// PSC_SR - Status Register : Read-only for UART mode				
#define PSC_USR_RB			BIT15		// 1 = Received Break
//#define PSC_USR_FE			BIT14		// 1 = Framing error detected
//#define PSC_USR_PE			BIT13		// 1 = Parity error	detected
//#define PSC_USR_ORERR		BIT12		// 1 = Overrun error detected
//#define PSC_USR_TXEMP		BIT11		// 1 = Transmitter empty
#define PSC_USR_TXEMP		0x0800		// 1 = Transmitter empty
//#define PSC_USR_CDE			BIT7		// 1 = No DCD error
//#define PSC_USR_ERROR		BIT6		// 1 = Error status detected
//#define PSC_USR_TIMEOUT		BIT5		// 1 = Time out event detected

// PSC_SR - Status Register : Read-only for other modes				
#define PSC_OSR_ORERR		BIT12		// 1 = Overrun error detected
#define PSC_OSR_URERR		BIT11		// 1 = Underrun error detected
#define PSC_OSR_ERROR		BIT6		// 1 = Error status detected
#define PSC_OSR_CMD_SEND	BIT3		// 1 = Command send ready
#define PSC_OSR_DATA_OVR	BIT2		// 1 = Receive status data overwrite detected
#define PSC_OSR_DATA_VALID	BIT1		// 1 = Receive Status Data valid
#define PSC_OSR_U_RX_SLOT	BIT0		// 1 = Unexpected receive slot detected
			
// PSC_CSR - Clock Select Register : UART mode only 				
#define PSC_RCS(_x_)	((_x_ & 0xf) << 12)	// Receive Clock Select Register
#define PSC_TCS(_x_)	((_x_ & 0xf) << 8)	// Transmitter Clock Select Register

// PSC_CR - Command Register 				
#define PSC_MISC(_x_)	((_x_ & 0x7) << 4)	// NISC field
#define PSC_TC(_x_)		((_x_ & 0x3) << 2)	// Transmitter Field
#define PSC_RC(_x_)		((_x_ & 0x3) << 0)	// Receiver field

// PSC_RB - Rx Buffer Register
#define PSC_RX_BUFF(_x_)		((_x_ & 0xffffffff) << 0)	// Receive Buffer

// PSC_TB - Tx Buffer Register
#define PSC_TX_BUFF(_x_)		((_x_ & 0xffffffff) << 0)	// Transmitter Buffer

// PSC_CTUR - Counter Timer Upper Register
//#define PSC_CTUR_UPPER(_x_)		((_x_ & 0xf) << 0)	// baudrate prescaler value in UART/SPI mode

// PSC_CTLR - Counter Timer Lower Register
//#define PSC_CTLR_LOWER(_x_)		((_x_ & 0xf) << 0)	// Baudrate prescaler value in UART mode

// PSC_SICR - Serial Interface Control Register 				
#define PSC_SPI			BIT23		// 1 = PSC behaves as SPI in Codec mode
#define PSC_MSTR		BIT22		// 1 = PSC behaves as SPI Master in Codec mode
#define PSC_CPOL		BIT21		// 1 = SPI Clock is Active-high	in Codec mode
#define PSC_CPHA		BIT20		// 1 = SPI Clock starts with first edge of SCK in Codec mode
#define PSC_EOF			BIT19		// 1 = Use End-of_Frame flag in Codec mode
#define PSC_OB_EN		BIT16		// 1 = Output Buffer enabled
#define PSC_GEN_CLK		BIT15		// 1 = Generate Bit Clock and FrameSync in Codec mode
#define PSC_I2S			BIT14		// 1 = PSC works in Codec I2S mode
#define PSC_CLK_POL		BIT13		// 1 = data sampled on rising and out on falling edge of BCLK
#define PSC_SNC_POL		BIT12		// 1 = FrameSync is high true in Codec SPI mode, Frame start LRCK high in I2S mode
#define PSC_ESAI		BIT9		// 1 = PSC supports Enhanced Serail Audio Interface in Codec mode
#define PSC_AC97_EN		BIT8		// 1 = AC97 tx and rx the data, only if AC97 selected
#define PSC_ACRB		BIT7		// 1 = Transmitter stays in current mode, AC97 mode
#define PSC_AWR			BIT6		// 1 = Force 1 on RTS output/AC97 Framesync, AC97 mode
#define PSC_DTS1		BIT5		// 1 = Delay of 1 clock cycle, Codec mode
#define PSC_SHDIR		BIT4		// 1 = shift direction is LSB
#define PSC_SIM(_x_)			((_x_ & 0xf) << 0)	// PSC operation mode

//-------------------------------------------------------------------------
// FIFOC - PSC Centralized FIFO Controller				
//-------------------------------------------------------------------------

#define PSC3_TX_CMD		0x80		// 
#define PSC3_RX_CMD		0xC0		// 
#define PSC3_TX_IMR		0x90		//  				
#define PSC3_RX_IMR		0xD0		//  				
#define PSC3_TX_SIZE	0x9C		//  				
#define PSC3_RX_SIZE	0xDC		//  				
#define PSC3_TX_STAT	0x88		//  				
#define PSC3_RX_STAT	0xC8		//  				
#define PSC3_TX_DATA	0xBC		//  				
#define PSC3_RX_DATA	0xFC		//  				



















