/* portInit():
 * Initialize the GPIO pins as they are used on this board.
 * This function is included for build with standard uMon,
 * mode3 and mode7 builds.
 *
 * SPI SELn Connections:
 * SPI_SEL1: Atmel Serial Flash (at25df321)
 * SPI_SEL2: ADI DAC (ad5384)
 * SPI_SEL3: ADI ADC (ad7490, IC4)
 * SPI_SEL4: J5 pin 6
 * SPI_SEL5: J5 pin 8
 * SPI_SEL6: J5 pin 9
 * SPI_SEL7: J5 pin 10
 */
void
portInit(void)
{
	volatile unsigned short stmp;

	/* Enable SPI SEL2-7:
	 */
	*pPORT_MUX = PFS4E | PFS5E | PFS6E | PJSE | PJCE_SPI;

	/* Port F configuration:
	 * (if not specified, assume no connect)
	 * PF0:  UART0_TX
	 * PF1:  UART0_RX
	 * PF4:  SPI_SEL_6
	 * PF5:  SPI_SEL_5
	 * PF6:  SPI_SEL_4
	 * PF10: SPI Flash Chip Select
	 * PF11: SPI Output
	 * PF12: SPI Input
	 * PF13: SPI Clock
	 * PF14: Pulled high
	 */
	stmp = *pPORTF_FER;
	*pPORTF_FER = 0x7c73;	/* Set the bits that are NOT used as GPIO */
	*pPORTF_FER = 0x7c73;
	*pPORTFIO_INEN = 0;

	/* Port G configuration:
	 * (if not specified, assume no connect)
	 * PG0:  AD5384_LDAC_N (GPIO output)
	 * PG1:  TOSA_TEC_EN (GPIO output, only used on TOSA)
	 *       (It is a no-connect for ROSA, so we can configure it for both)
	 * PG6:  UART_CTS
	 * PG7:  UART_RTS
	 */
	stmp = *pPORTG_FER;
	*pPORTG_FER = 0x00c0; 	/* Set the bits that are NOT used as GPIO */
	*pPORTG_FER = 0x00c0;

	*pPORTGIO_CLEAR = AD5384_LDAC_N_BIT;
	*pPORTGIO_SET = TOSA_TEC_EN_BIT;
	*pPORTGIO_DIR = AD5384_LDAC_N_BIT | TOSA_TEC_EN_BIT;
	*pPORTGIO_INEN = 0;

	/* Port H configuration:
	 * (dedicated to ethernet)
	 */
	stmp = *pPORTH_FER;
	*pPORTH_FER = 0xffff;
	*pPORTH_FER = 0xffff;
}
