/* tfsdev.h:
	This file is ONLY included by tfs.c.  It is seperate from tfs.h because
	it is target-specific.  It is not part of config.h because it includes
	the declaration of the tfsdevtbl[].
	A prefix in the name of the file determines what device is used to store
	that file.  If no prefix is found the the first device in the table is
	used as a default.  The syntax of the prefix is "//STRING/" where STRING
	is user-definable, but the initial // and final / are required by tfs
	code.
*/
struct tfsdev tfsdevtbl[] = {
#if INCLUDE_FLASH
	{
		"//FLASH/",
		TFSSTART,
		TFSEND,
		TFSSPARE,
		TFSSPARESIZE,
		TFSSECTORCOUNT,
		TFS_DEVTYPE_FLASH },
#endif
	
#if INCLUDE_TFSRAM
	/* This RAM section is preloaded by the SPI (mode3) boot.
	 * As a result, it  must be configured as NVRAM so that TFS doesn't
	 * automatically clear it to all FFs...
	 */
	{
		"//RAM/",
		0x3800000,
		0x3aeffff,
		0x3af0000,
		0x10000,
		47,
		TFS_DEVTYPE_NVRAM },
#endif
	{ 0, TFSEOT,0,0,0,0,0 }
};

#define TFSDEVTOT ((sizeof(tfsdevtbl))/(sizeof(struct tfsdev)))
